@@ -42,7 +42,9 @@ release_cArray (char **carray)
   if (carray)
     {
       for (int idx = 0; carray[idx]; idx++)
-        xfree (carray[idx]);
+        {
+          xfree (carray[idx]);
+        }
       xfree (carray);
     }
 }
@@ -76,7 +78,7 @@ vector_to_cArray(const std::vector<std::string> &vec)
   char ** ret = (char**) xmalloc (sizeof (char*) * (vec.size() + 1));
   for (size_t i = 0; i < vec.size(); i++)
     {
-      ret[i] = strdup (vec[i].c_str());
+      ret[i] = xstrdup (vec[i].c_str());
     }
   ret[vec.size()] = NULL;
   return ret;
@@ -112,6 +114,7 @@ in_de_vs_mode()
     {
       return vs_mode;
     }
+  checked = true;
   GpgME::Error err;
   const auto components = GpgME::Configuration::Component::load (err);
   log_debug ("%s:%s: Checking for de-vs mode.",
@@ -120,7 +123,6 @@ in_de_vs_mode()
     {
       log_error ("%s:%s: Failed to get gpgconf components: %s",
                  SRCNAME, __func__, err.asString ());
-      checked = true;
       vs_mode = false;
       return vs_mode;
     }
@@ -136,17 +138,14 @@ in_de_vs_mode()
                 {
                   log_debug ("%s:%s: Detected de-vs mode",
                              SRCNAME, __func__);
-                  checked = true;
                   vs_mode = true;
                   return vs_mode;
                 }
             }
-          checked = true;
           vs_mode = false;
           return vs_mode;
         }
     }
-  checked = true;
   vs_mode = false;
   return false;
 }
@@ -257,3 +256,47 @@ gpgol_split (const std::string &s, char delim)
   internal_split (s, delim, std::back_inserter (elems));
   return elems;
 }
+
+std::string
+string_to_hex(const std::string& input)
+{
+    static const char* const lut = "0123456789ABCDEF";
+    size_t len = input.length();
+
+    std::string output;
+    output.reserve (3 * len + (len * 3 / 26));
+    for (size_t i = 0; i < len; ++i)
+    {
+        const unsigned char c = input[i];
+        output.push_back (lut[c >> 4]);
+        output.push_back (lut[c & 15]);
+        output.push_back (' ');
+        if (i % 26 == 0)
+          {
+            output.push_back ('\n');
+          }
+    }
+    return output;
+}
+
+bool
+is_binary (const std::string &input)
+{
+  for (int i = 0; i < input.size() - 1; ++i)
+    {
+      const unsigned char c = input[i];
+      if (c < 32 && c != 0x0d && c != 0x0a)
+        {
+          return true;
+        }
+    }
+  return false;
+}
+
+const char *
+to_cstr (const GpgME::Protocol &prot)
+{
+  return prot == GpgME::CMS ? "S/MIME" :
+         prot == GpgME::OpenPGP ? "OpenPGP" :
+         "Unknown Protocol";
+}
