@@ -68,16 +68,17 @@ CryptController::CryptController (Mail *mail, bool encrypt, bool sign,
     m_crypto_success (false),
     m_proto (proto)
 {
+  memdbg_ctor ("CryptController");
   log_debug ("%s:%s: CryptController ctor for %p encrypt %i sign %i inline %i.",
-             SRCNAME, __func__, mail, encrypt, sign, mail->do_pgp_inline ());
-  m_recipient_addrs = mail->take_cached_recipients ();
+             SRCNAME, __func__, mail, encrypt, sign, mail->getDoPGPInline ());
+  m_recipient_addrs = mail->getCachedRecipients ();
 }
 
 CryptController::~CryptController()
 {
+  memdbg_dtor ("CryptController");
   log_debug ("%s:%s:%p",
              SRCNAME, __func__, m_mail);
-  release_cArray (m_recipient_addrs);
 }
 
 int
@@ -94,7 +95,7 @@ CryptController::collect_data ()
   /* Take the Body from the mail if possible. This is a fix for
      GnuPG-Bug-ID: T3614 because the body is not always properly
      updated in MAPI when sending. */
-  char *body = m_mail->take_cached_plain_body ();
+  char *body = m_mail->takeCachedPlainBody ();
   if (body && !*body)
     {
       xfree (body);
@@ -112,15 +113,16 @@ CryptController::collect_data ()
   int n_att_usable = count_usable_attachments (att_table);
   if (!n_att_usable && !body)
     {
-      gpgol_message_box (m_mail->get_window(),
+      gpgol_message_box (m_mail->getWindow (),
                          utf8_gettext ("Can't encrypt / sign an empty message."),
                          utf8_gettext ("GpgOL"), MB_OK);
       gpgol_release (message);
+      mapi_release_attach_table (att_table);
       xfree (body);
       return -1;
     }
 
-  bool do_inline = m_mail->do_pgp_inline ();
+  bool do_inline = m_mail->getDoPGPInline ();
 
   if (n_att_usable && do_inline)
     {
@@ -128,7 +130,7 @@ CryptController::collect_data ()
                  " Using PGP MIME",
                  SRCNAME, __func__);
       do_inline = false;
-      m_mail->set_do_pgp_inline (false);
+      m_mail->setDoPGPInline (false);
     }
   else if (do_inline)
     {
@@ -150,18 +152,24 @@ CryptController::collect_data ()
   sink->writefnc = sink_data_write;
 
   /* Collect the mime strucutre */
-  if (add_body_and_attachments (sink, message, att_table, m_mail,
-                                body, n_att_usable))
+  int err = add_body_and_attachments (sink, message, att_table, m_mail,
+                                      body, n_att_usable);
+  xfree (body);
+
+  if (err)
     {
       log_error ("%s:%s: Collecting body and attachments failed.",
                  SRCNAME, __func__);
       gpgol_release (message);
+      mapi_release_attach_table (att_table);
       return -1;
     }
 
   /* Message is no longer needed */
   gpgol_release (message);
 
+  mapi_release_attach_table (att_table);
+
   /* Set the input buffer to start. */
   m_input.seek (0, SEEK_SET);
   return 0;
@@ -213,6 +221,7 @@ CryptController::lookup_fingerprints (const std::string &sigFpr,
   if (err) {
       log_error ("%s:%s: failed to start recipient keylisting",
                  SRCNAME, __func__);
+      release_cArray (cRecps);
       return -1;
   }
 
@@ -295,7 +304,7 @@ CryptController::parse_output (GpgME::Data &resolverOutput)
     {
       log_error ("%s:%s: Encrypt requested but no recipient fingerprints",
                  SRCNAME, __func__);
-      gpgol_message_box (m_mail->get_window(), _("No recipients for encryption selected."),
+      gpgol_message_box (m_mail->getWindow (), _("No recipients for encryption selected."),
                          _("GpgOL"), MB_OK);
       return -2;
     }
@@ -303,59 +312,105 @@ CryptController::parse_output (GpgME::Data &resolverOutput)
   return lookup_fingerprints (sigFpr, recpFprs);
 }
 
+static bool
+resolve_through_protocol (const GpgME::Protocol &proto, bool sign,
+                          bool encrypt, const std::string &sender,
+                          const std::vector<std::string> &recps,
+                          std::vector<GpgME::Key> &r_keys,
+                          GpgME::Key &r_sig)
+{
+  bool sig_ok = true;
+  bool enc_ok = true;
+
+  const auto cache = KeyCache::instance();
+
+  if (encrypt)
+    {
+      r_keys = cache->getEncryptionKeys(recps, proto);
+      enc_ok = !r_keys.empty();
+    }
+  if (sign && enc_ok)
+    {
+      r_sig = cache->getSigningKey (sender.c_str (), proto);
+      sig_ok = !r_sig.isNull();
+    }
+  return sig_ok && enc_ok;
+}
+
 int
 CryptController::resolve_keys_cached()
 {
-  const auto cache = KeyCache::instance();
-
-  bool fallbackToSMIME = false;
+  // Prepare variables
+  const auto cached_sender = m_mail->getSender ();
+  auto recps = m_recipient_addrs;
 
   if (m_encrypt)
     {
-      const auto cached_sender = m_mail->get_cached_sender ();
-      auto recps = cArray_to_vector ((const char**) m_recipient_addrs);
       recps.push_back (cached_sender);
+    }
 
-      m_recipients = cache->getEncryptionKeys(recps, GpgME::OpenPGP);
-      m_proto = GpgME::OpenPGP;
-
-      if (m_recipients.empty() && opt.enable_smime)
+  bool resolved = false;
+  if (opt.enable_smime && opt.prefer_smime)
+    {
+      resolved = resolve_through_protocol (GpgME::CMS, m_sign, m_encrypt,
+                                           cached_sender, recps, m_recipients,
+                                           m_signer_key);
+      if (resolved)
         {
-          m_recipients = cache->getEncryptionKeys(recps, GpgME::CMS);
-          fallbackToSMIME = true;
+          log_debug ("%s:%s: Resolved with CMS due to preference.",
+                     SRCNAME, __func__);
           m_proto = GpgME::CMS;
         }
-      if (m_recipients.empty())
+    }
+  if (!resolved)
+    {
+      resolved = resolve_through_protocol (GpgME::OpenPGP, m_sign, m_encrypt,
+                                           cached_sender, recps, m_recipients,
+                                           m_signer_key);
+      if (resolved)
         {
-          log_debug ("%s:%s: Failed to resolve keys through cache",
+          log_debug ("%s:%s: Resolved with OpenPGP.",
                      SRCNAME, __func__);
-          m_proto = GpgME::UnknownProtocol;
-          return 1;
+          m_proto = GpgME::OpenPGP;
+        }
+    }
+  if (!resolved && (opt.enable_smime && !opt.prefer_smime))
+    {
+      resolved = resolve_through_protocol (GpgME::CMS, m_sign, m_encrypt,
+                                           cached_sender, recps, m_recipients,
+                                           m_signer_key);
+      if (resolved)
+        {
+          log_debug ("%s:%s: Resolved with CMS as fallback.",
+                     SRCNAME, __func__);
+          m_proto = GpgME::CMS;
         }
     }
 
-  if (m_sign)
+  if (!resolved)
     {
-      if (!fallbackToSMIME)
-        {
-          m_signer_key = cache->getSigningKey (m_mail->get_cached_sender ().c_str (),
-                                               GpgME::OpenPGP);
-          m_proto = GpgME::OpenPGP;
-        }
-      if (m_signer_key.isNull() && opt.enable_smime)
-        {
-          m_signer_key = cache->getSigningKey (m_mail->get_cached_sender ().c_str (),
-                                               GpgME::CMS);
-          m_proto = GpgME::CMS;
-        }
-      if (m_signer_key.isNull())
-        {
-          log_debug ("%s:%s: Failed to resolve signer key through cache",
-                     SRCNAME, __func__);
-          m_recipients.clear();
-          m_proto = GpgME::UnknownProtocol;
-          return 1;
-        }
+      log_debug ("%s:%s: Failed to resolve through cache",
+                 SRCNAME, __func__);
+      m_recipients.clear();
+      m_signer_key = GpgME::Key();
+      m_proto = GpgME::UnknownProtocol;
+      return 1;
+    }
+
+  if (!m_recipients.empty())
+    {
+      log_debug ("%s:%s: Encrypting with protocol %s to:",
+                 SRCNAME, __func__, to_cstr (m_proto));
+    }
+  for (const auto &key: m_recipients)
+    {
+      log_debug ("%s", key.primaryFingerprint ());
+    }
+  if (!m_signer_key.isNull())
+    {
+      log_debug ("%s:%s: Signing key: %s:%s",
+                 SRCNAME, __func__, m_signer_key.primaryFingerprint (),
+                 to_cstr (m_signer_key.protocol()));
     }
   return 0;
 }
@@ -392,7 +447,7 @@ CryptController::resolve_keys ()
   args.push_back (std::string ("--debug"));
 
   // Yes passing it as int is ok.
-  auto wnd = m_mail->get_window ();
+  auto wnd = m_mail->getWindow ();
   if (wnd)
     {
       // Pass the handle of the active window for raise / overlay.
@@ -421,7 +476,7 @@ CryptController::resolve_keys ()
     {
       args.push_back (std::string ("--sign"));
     }
-  const auto cached_sender = m_mail->get_cached_sender ();
+  const auto cached_sender = m_mail->getSender ();
   if (cached_sender.empty())
     {
       log_error ("%s:%s: resolve keys without sender.",
@@ -443,9 +498,9 @@ CryptController::resolve_keys ()
     {
       args.push_back (std::string ("--encrypt"));
       // Get the recipients that are cached from OOM
-      for (size_t i = 0; m_recipient_addrs && m_recipient_addrs[i]; i++)
+      for (const auto &addr: m_recipient_addrs)
         {
-          args.push_back (GpgME::UserID::addrSpecFromString (m_recipient_addrs[i]));
+          args.push_back (GpgME::UserID::addrSpecFromString (addr.c_str()));
         }
     }
 
@@ -520,7 +575,7 @@ CryptController::do_crypto (GpgME::Error &err)
              SRCNAME, __func__);
 
   /* Start a WKS check if necessary. */
-  WKSHelper::instance()->start_check (m_mail->get_cached_sender ());
+  WKSHelper::instance()->start_check (m_mail->getSender ());
 
   int ret = resolve_keys ();
   if (ret == -1)
@@ -535,14 +590,14 @@ CryptController::do_crypto (GpgME::Error &err)
       // Cancel
       return -2;
     }
-  bool do_inline = m_mail->do_pgp_inline ();
+  bool do_inline = m_mail->getDoPGPInline ();
 
   if (m_proto == GpgME::CMS && do_inline)
     {
       log_debug ("%s:%s: Inline for S/MIME not supported. Switching to mime.",
                  SRCNAME, __func__);
       do_inline = false;
-      m_mail->set_do_pgp_inline (false);
+      m_mail->setDoPGPInline (false);
       m_bodyInput = GpgME::Data(GpgME::Data::null);
     }
 
@@ -552,7 +607,7 @@ CryptController::do_crypto (GpgME::Error &err)
     {
       log_error ("%s:%s: Failure to create context.",
                  SRCNAME, __func__);
-      gpgol_message_box (m_mail->get_window (),
+      gpgol_message_box (m_mail->getWindow (),
                          "Failure to create context.",
                          utf8_gettext ("GpgOL"), MB_OK);
       return -1;
@@ -892,7 +947,7 @@ CryptController::update_mail_mapi ()
       return -1;
     }
 
-  if (m_mail->do_pgp_inline ())
+  if (m_mail->getDoPGPInline ())
     {
       // Nothing to do for inline.
       log_debug ("%s:%s: Inline mail. Setting encoding.",
@@ -926,7 +981,7 @@ CryptController::update_mail_mapi ()
   // This means that the conversion / build of the mime structure also
   // happens differently.
   int exchange_major_version = get_ex_major_version_for_addr (
-                                        m_mail->get_cached_sender ().c_str ());
+                                        m_mail->getSender ().c_str ());
 
   std::string overrideMimeTag;
   if (m_proto == GpgME::CMS && m_encrypt && exchange_major_version >= 15)
@@ -1002,7 +1057,7 @@ std::string
 CryptController::get_inline_data ()
 {
   std::string ret;
-  if (!m_mail->do_pgp_inline ())
+  if (!m_mail->getDoPGPInline ())
     {
       return ret;
     }
@@ -1054,7 +1109,7 @@ CryptController::parse_micalg (const GpgME::SigningResult &result)
 void
 CryptController::start_crypto_overlay ()
 {
-  auto wid = m_mail->get_window ();
+  auto wid = m_mail->getWindow ();
 
   std::string text;
 
