@@ -46,6 +46,8 @@
 #include "windowmessages.h"
 #include "mail.h"
 #include "addin-options.h"
+#include "cpphelp.h"
+#include "dispcache.h"
 
 #include <gpg-error.h>
 #include <list>
@@ -102,6 +104,7 @@ STDAPI DllCanUnloadNow()
      unload the Library. Any callbacks will become invalid.
      So we _only_ say it's ok to unload if we were disconnected.
      For the epic story behind the next line see GnuPG-Bug-Id 1837 */
+  TRACEPOINT;
   return can_unload ? S_OK : S_FALSE;
 }
 
@@ -153,6 +156,11 @@ STDMETHODIMP GpgolAddinFactory::CreateInstance (LPUNKNOWN punk, REFIID riid,
   return hr;
 }
 
+GpgolAddinFactory::~GpgolAddinFactory()
+{
+  log_debug ("%s:%s: Object deleted\n", SRCNAME, __func__);
+}
+
 /* GpgolAddin definition */
 
 
@@ -170,7 +178,9 @@ GpgolAddin::GpgolAddin (void) : m_lRef(0),
   m_applicationEventSink(nullptr),
   m_explorersEventSink(nullptr),
   m_disabled(false),
-  m_hook(nullptr)
+  m_shutdown(false),
+  m_hook(nullptr),
+  m_dispcache(new DispCache)
 {
   read_options ();
   /* RibbonExtender is it's own object to avoid the pitfalls of
@@ -185,15 +195,6 @@ GpgolAddin::~GpgolAddin (void)
     {
       return;
     }
-  log_debug ("%s:%s: Releasing Application Event Sink;",
-             SRCNAME, __func__);
-  gpgol_release (m_explorersEventSink);
-  gpgol_release (m_applicationEventSink);
-
-  write_options ();
-
-  UnhookWindowsHookEx (m_hook);
-
   addin_instance = NULL;
 
   log_debug ("%s:%s: Object deleted\n", SRCNAME, __func__);
@@ -316,9 +317,11 @@ GpgolAddin::OnConnection (LPDISPATCH Application, ext_ConnectMode ConnectMode,
   log_debug ("%s:%s: this is GpgOL %s\n",
              SRCNAME, __func__, PACKAGE_VERSION);
 
+  m_shutdown = false;
   can_unload = false;
   m_application = Application;
   m_application->AddRef();
+  memdbg_addRef (m_application);
   m_addin = AddInInst;
 
   version = get_oom_string (Application, "Version");
@@ -343,6 +346,7 @@ GpgolAddin::OnConnection (LPDISPATCH Application, ext_ConnectMode ConnectMode,
       return S_OK;
     }
 
+  xfree (version);
   setupDoNotDisable ();
 
   if (ConnectMode != ext_cm_Startup)
@@ -365,18 +369,7 @@ GpgolAddin::OnDisconnection (ext_DisconnectMode RemoveMode,
      does not allow us any OOM calls then and only returns
      "Unexpected error" in that case. Weird. */
 
-  if (Mail::close_all_mails ())
-    {
-      MessageBox (NULL,
-                  "Failed to remove plaintext from at least one message.\n\n"
-                  "Until GpgOL is activated again it is possible that the "
-                  "plaintext of messages decrypted in this Session is saved "
-                  "or transfered back to your mailserver.",
-                  _("GpgOL"),
-                  MB_ICONINFORMATION|MB_OK);
-    }
-
-  write_options();
+  shutdown ();
   can_unload = true;
   return S_OK;
 }
@@ -469,12 +462,27 @@ install_explorer_sinks (LPDISPATCH application)
         {
           log_oom_extra ("%s:%s: created sink %p for explorer %i",
                          SRCNAME, __func__, sink, i);
+          GpgolAddin::get_instance ()->registerExplorerSink (sink);
         }
-      add_explorer (explorer);
       gpgol_release (explorer);
     }
   /* Now install the event sink to handle new explorers */
-  return install_ExplorersEvents_sink (explorers);
+  LPDISPATCH ret = install_ExplorersEvents_sink (explorers);
+  gpgol_release (explorers);
+
+  return ret;
+}
+
+static DWORD WINAPI
+init_gpgme_config (LPVOID)
+{
+  /* This is a check we need to do anyway. GpgME++ caches
+     the configuration once it is accessed for the first time
+     so this call also initializes GpgME++ */
+  bool de_vs_mode = in_de_vs_mode ();
+  log_debug ("%s:%s: init_gpgme_config de_vs_mode %i",
+             SRCNAME, __func__, de_vs_mode);
+  return 0;
 }
 
 STDMETHODIMP
@@ -483,6 +491,8 @@ GpgolAddin::OnStartupComplete (SAFEARRAY** custom)
   (void)custom;
   TRACEPOINT;
 
+  i18n_init ();
+
   if (!create_responder_window())
     {
       log_error ("%s:%s: Failed to create the responder window;",
@@ -512,6 +522,9 @@ GpgolAddin::OnStartupComplete (SAFEARRAY** custom)
   m_applicationEventSink = install_ApplicationEvents_sink (m_application);
   m_explorersEventSink = install_explorer_sinks (m_application);
   check_html_preferred ();
+
+  CloseHandle (CreateThread (NULL, 0, init_gpgme_config, nullptr, 0,
+                             NULL));
   return S_OK;
 }
 
@@ -578,7 +591,7 @@ GpgolRibbonExtender::~GpgolRibbonExtender (void)
 {
   log_debug ("%s:%s: cleaning up GpgolRibbonExtender object;",
              SRCNAME, __func__);
-  log_debug ("%s:%s: Object deleted\n", SRCNAME, __func__);
+  memdbg_dump ();
 }
 
 STDMETHODIMP
@@ -1059,7 +1072,8 @@ GetCustomUI_MIME (BSTR RibbonID, BSTR * RibbonXml)
 
   if (buffer)
     {
-      wchar_t *wbuf = utf8_to_wchar2 (buffer, strlen(buffer));
+      memdbg_alloc (buffer);
+      wchar_t *wbuf = utf8_to_wchar (buffer);
       xfree (buffer);
       *RibbonXml = SysAllocString (wbuf);
       xfree (wbuf);
@@ -1107,5 +1121,80 @@ GpgolAddin::get_instance ()
     {
       addin_instance = new GpgolAddin ();
     }
+  if (addin_instance->isShutdown ())
+    {
+      log_error ("%s:%s: Get instance after shutdown",
+                 SRCNAME, __func__);
+    }
   return addin_instance;
 }
+
+void
+GpgolAddin::shutdown ()
+{
+  if (m_shutdown)
+    {
+      log_debug ("%s:%s: Already shutdown",
+                 SRCNAME, __func__);
+      return;
+    }
+
+  /* Disabling message hook */
+  UnhookWindowsHookEx (m_hook);
+
+  log_debug ("%s:%s: Releasing Application Event Sink;",
+             SRCNAME, __func__);
+  detach_ApplicationEvents_sink (m_applicationEventSink);
+  gpgol_release (m_applicationEventSink);
+
+  log_debug ("%s:%s: Releasing Explorers Event Sink;",
+             SRCNAME, __func__);
+  detach_ExplorersEvents_sink (m_explorersEventSink);
+  gpgol_release (m_explorersEventSink);
+
+  log_debug ("%s:%s: Releasing Explorer Event Sinks;",
+             SRCNAME, __func__);
+  for (auto sink: m_explorerEventSinks)
+    {
+      detach_ExplorerEvents_sink (sink);
+      gpgol_release (sink);
+    }
+
+  write_options ();
+
+  if (Mail::closeAllMails_o ())
+    {
+      MessageBox (NULL,
+                  "Failed to remove plaintext from at least one message.\n\n"
+                  "Until GpgOL is activated again it is possible that the "
+                  "plaintext of messages decrypted in this Session is saved "
+                  "or transfered back to your mailserver.",
+                  _("GpgOL"),
+                  MB_ICONINFORMATION|MB_OK);
+    }
+  m_shutdown = true;
+
+  gpgol_release (m_application);
+  m_application = nullptr;
+}
+
+void
+GpgolAddin::registerExplorerSink (LPDISPATCH sink)
+{
+  m_explorerEventSinks.push_back (sink);
+}
+
+void
+GpgolAddin::unregisterExplorerSink (LPDISPATCH sink)
+{
+  for (int i = 0; i < m_explorerEventSinks.size(); ++i)
+    {
+      if (m_explorerEventSinks[i] == sink)
+        {
+          m_explorerEventSinks.erase(m_explorerEventSinks.begin() + i);
+          return;
+        }
+    }
+  log_error ("%s:%s: Unregister %p which was not registered.",
+             SRCNAME, __func__, sink);
+}
