@@ -34,6 +34,8 @@ namespace GpgME
   class Key;
 };
 
+class Mail;
+
 class KeyCache
 {
 protected:
@@ -57,22 +59,50 @@ public:
                                                GpgME::Protocol proto) const;
 
     /* Start a key location in a background thread filling
-       the key cache. cArray is a null terminated array
-       of address strings. */
-    void startLocate (char **cArray) const;
+       the key cache.
+
+       The mail argument is used to add / remove the
+       locator thread counter.
+       */
+    void startLocate (const std::vector<std::string> &addrs, Mail *mail) const;
 
     /* Look for a secret key for the addr. */
-    void startLocateSecret (const char *addr) const;
+    void startLocateSecret (const char *addr, Mail *mail) const;
 
     /* Start a key location in a background thread filling
        the key cache. */
-    void startLocate (const char *addr) const;
+    void startLocate (const char *addr, Mail *mail) const;
+
+    /* Check that a mail is resolvable through the keycache.
+     *
+     * For OpenPGP only the recipients are checked as we can
+     * generate a new key for the sender.
+     **/
+    bool isMailResolvable (Mail *mail);
+
+    /* Search / Update a key in the cache. This is meant to be
+       called e.g. after a verify to update the key.
+
+       A known issue is that a get right after it might
+       still return an outdated key but the get after that
+       would return the updated one. This is acceptable as
+       it only poses a minor problem with TOFU while we
+       can show the correct state in the tooltip. */
+    void update (const char *fpr, GpgME::Protocol proto);
+
+    /* Get a cached key. If block is true it will block
+       if the key is currently searched for.
+
+       This function will not search a key. Call update
+       to insert keys into the cache */
+    GpgME::Key getByFpr (const char *fpr, bool block = true) const;
 
     // Internal for thread
-    void setSmimeKey(const char *mbox, const GpgME::Key &key);
-    void setPgpKey(const char *mbox, const GpgME::Key &key);
-    void setSmimeKeySecret(const char *mbox, const GpgME::Key &key);
-    void setPgpKeySecret(const char *mbox, const GpgME::Key &key);
+    void setSmimeKey(const std::string &mbox, const GpgME::Key &key);
+    void setPgpKey(const std::string &mbox, const GpgME::Key &key);
+    void setSmimeKeySecret(const std::string &mbox, const GpgME::Key &key);
+    void setPgpKeySecret(const std::string &mbox, const GpgME::Key &key);
+    void onUpdateJobDone (const char *fpr, const GpgME::Key &key);
 
 private:
 
