@@ -30,6 +30,7 @@
 #include "mail.h"
 #include "mapihelp.h"
 #include "gpgoladdin.h"
+#include "wks-helper.h"
 
 #undef _
 #define _(a) utf8_gettext (a)
@@ -127,7 +128,7 @@ EVENT_SINK_INVOKE(MailItemEvents)
   USE_INVOKE_ARGS
   if (!m_mail)
     {
-      m_mail = Mail::get_mail_for_item (m_object);
+      m_mail = Mail::getMailForItem (m_object);
       if (!m_mail)
         {
           log_error ("%s:%s: mail event without mail object known. Bug.",
@@ -143,23 +144,12 @@ EVENT_SINK_INVOKE(MailItemEvents)
         {
           log_oom_extra ("%s:%s: Open : %p",
                          SRCNAME, __func__, m_mail);
-          LPMESSAGE message;
-          if (g_ol_version_major < 14 && m_mail->set_uuid ())
-            {
-              /* In Outlook 2007 we need the uid for every
-                 open mail to track the message in case
-                 it is sent and crypto is required. */
-              log_debug ("%s:%s: Failed to set uuid.",
-                         SRCNAME, __func__);
-              delete m_mail; /* deletes this, too */
-              return S_OK;
-            }
           int draft_flags = 0;
           if (!opt.encrypt_default && !opt.sign_default)
             {
               return S_OK;
             }
-          message = get_oom_base_message (m_object);
+          LPMESSAGE message = get_oom_base_message (m_object);
           if (!message)
             {
               log_error ("%s:%s: Failed to get message.",
@@ -182,7 +172,7 @@ EVENT_SINK_INVOKE(MailItemEvents)
         {
           log_oom_extra ("%s:%s: BeforeRead : %p",
                          SRCNAME, __func__, m_mail);
-          if (m_mail->pre_process_message ())
+          if (m_mail->preProcessMessage_m ())
             {
               log_error ("%s:%s: Pre process message failed.",
                          SRCNAME, __func__);
@@ -193,35 +183,35 @@ EVENT_SINK_INVOKE(MailItemEvents)
         {
           log_oom_extra ("%s:%s: Read : %p",
                          SRCNAME, __func__, m_mail);
-          if (!m_mail->is_crypto_mail())
+          if (!m_mail->isCryptoMail ())
             {
               log_debug ("%s:%s: Non crypto mail %p opened. Updating sigstatus.",
                          SRCNAME, __func__, m_mail);
               /* Ensure that no wrong sigstatus is shown */
-              CloseHandle(CreateThread (NULL, 0, delayed_invalidate_ui, (LPVOID) this, 0,
+              CloseHandle(CreateThread (NULL, 0, delayed_invalidate_ui, (LPVOID) 300, 0,
                                         NULL));
               break;
             }
-          if (m_mail->set_uuid ())
+          if (m_mail->setUUID_o ())
             {
               log_debug ("%s:%s: Failed to set uuid.",
                          SRCNAME, __func__);
               delete m_mail; /* deletes this, too */
               return S_OK;
             }
-          if (m_mail->decrypt_verify ())
+          if (m_mail->decryptVerify_o ())
             {
               log_error ("%s:%s: Decrypt message failed.",
                          SRCNAME, __func__);
             }
-          if (!opt.enable_smime && m_mail->is_smime ())
+          if (!opt.enable_smime && m_mail->isSMIME_m ())
             {
               /* We want to save the mail when it's an smime mail and smime
                  is disabled to revert it. */
               log_debug ("%s:%s: S/MIME mail but S/MIME is disabled."
                          " Need save.",
                          SRCNAME, __func__);
-              m_mail->set_needs_save (true);
+              m_mail->setNeedsSave (true);
             }
           break;
         }
@@ -236,19 +226,24 @@ EVENT_SINK_INVOKE(MailItemEvents)
               break;
             }
           const wchar_t *prop_name = parms->rgvarg[0].bstrVal;
-          if (!m_mail->is_crypto_mail ())
+          if (!m_mail->isCryptoMail ())
             {
               if (!opt.autoresolve)
                 {
                   break;
                 }
+              if (m_mail->hasOverrideMimeData())
+                {
+                  /* This is a mail created by us. Ignore propchanges. */
+                  break;
+                }
               if (!wcscmp (prop_name, L"To") /* ||
                   !wcscmp (prop_name, L"BCC") ||
                   !wcscmp (prop_name, L"CC")
                   Testing shows that Outlook always sends these three in a row
                   */)
                 {
-                  if ((m_mail->needs_crypto() & 1))
+                  if (opt.autosecure || (m_mail->needs_crypto_m () & 1))
                     {
                       /* XXX Racy race. This is a fix for crashes
                          that happend if a resolved recipient is copied an pasted.
@@ -285,7 +280,7 @@ EVENT_SINK_INVOKE(MailItemEvents)
                 }
               log_debug ("%s:%s: Message %p looks like send again.",
                         SRCNAME, __func__, m_object);
-              m_mail->set_is_send_again (true);
+              m_mail->setIsSendAgain (true);
               return S_OK;
             }
 
@@ -320,6 +315,7 @@ EVENT_SINK_INVOKE(MailItemEvents)
                                   "message is not open in any window and not selected in the "
                                   "messagelist.\n\nFor example by right clicking but not selecting the message.\n"),
                           wchar_to_utf8(prop_name));
+          memdbg_alloc (fmt);
           wchar_t *msg = utf8_to_wchar (fmt);
           xfree (fmt);
           MessageBoxW (get_active_hwnd(), msg, title,
@@ -355,7 +351,7 @@ EVENT_SINK_INVOKE(MailItemEvents)
            */
           log_oom_extra ("%s:%s: Send : %p",
                          SRCNAME, __func__, m_mail);
-          if (!m_mail->needs_crypto () && m_mail->crypt_state () == Mail::NoCryptMail)
+          if (!m_mail->needs_crypto_m () && m_mail->cryptState () == Mail::NoCryptMail)
             {
              log_debug ("%s:%s: No crypto neccessary. Passing send for %p obj %p",
                         SRCNAME, __func__, m_mail, m_object);
@@ -369,23 +365,34 @@ EVENT_SINK_INVOKE(MailItemEvents)
              break;
            }
 
-          if (m_mail->crypt_state () == Mail::NoCryptMail &&
-              m_mail->needs_crypto ())
+          if (m_mail->cryptState () == Mail::NoCryptMail &&
+              m_mail->needs_crypto_m ())
             {
+              log_debug ("%s:%s: Send event for crypto mail %p saving and starting.",
+                         SRCNAME, __func__, m_mail);
+
+              if (!m_mail->isAsyncCryptDisabled())
+                {
+                  /* Obtain a reference of the current item. This prevents
+                   * an early unload which would crash Outlook 2013
+                   *
+                   * As it didn't crash when the mail was opened in Outlook Spy this
+                   * mimics that the mail is inspected somewhere else. */
+                  m_mail->refCurrentItem ();
+                }
+
               // First contact with a mail to encrypt update
               // state and oom data.
-              m_mail->update_oom_data ();
-              m_mail->set_crypt_state (Mail::NeedsFirstAfterWrite);
+              m_mail->updateOOMData_o ();
+
+              m_mail->setCryptState (Mail::NeedsFirstAfterWrite);
 
               // Check inline response state before the write.
               m_mail->check_inline_response ();
-
-              log_debug ("%s:%s: Send event for crypto mail %p saving and starting.",
-                         SRCNAME, __func__, m_mail);
               // Save the Mail
               invoke_oom_method (m_object, "Save", NULL);
 
-              if (!m_mail->async_crypt_disabled ())
+              if (!m_mail->isAsyncCryptDisabled ())
                 {
                   // The afterwrite in the save should have triggered
                   // the encryption. We cancel send for our asyncness.
@@ -395,7 +402,7 @@ EVENT_SINK_INVOKE(MailItemEvents)
                 }
               else
                 {
-                  if (m_mail->crypt_state () == Mail::NoCryptMail)
+                  if (m_mail->cryptState () == Mail::NoCryptMail)
                     {
                       // Crypto failed or was canceled
                       log_debug ("%s:%s: Message %p mail %p cancelling send - "
@@ -406,20 +413,20 @@ EVENT_SINK_INVOKE(MailItemEvents)
                     }
                   // For inline response we can't trigger send programatically
                   // so we do the encryption in sync.
-                  if (m_mail->crypt_state () == Mail::NeedsUpdateInOOM)
+                  if (m_mail->cryptState () == Mail::NeedsUpdateInOOM)
                     {
-                      m_mail->update_crypt_oom ();
+                      m_mail->updateCryptOOM_o ();
                     }
-                  if (m_mail->crypt_state () == Mail::NeedsSecondAfterWrite)
+                  if (m_mail->cryptState () == Mail::NeedsSecondAfterWrite)
                     {
-                      m_mail->set_crypt_state (Mail::WantsSendMIME);
+                      m_mail->setCryptState (Mail::WantsSendMIME);
                     }
-                  if (m_mail->do_pgp_inline () && m_mail->crypt_state () != Mail::WantsSendInline)
+                  if (m_mail->getDoPGPInline () && m_mail->cryptState () != Mail::WantsSendInline)
                     {
                       log_debug ("%s:%s: Message %p mail %p cancelling send - "
                                  "Invalid state.",
                                  SRCNAME, __func__, m_object, m_mail);
-                      gpgol_bug (m_mail->get_window (),
+                      gpgol_bug (m_mail->getWindow (),
                                  ERR_INLINE_BODY_INV_STATE);
                       *(parms->rgvarg[0].pboolVal) = VARIANT_TRUE;
                       break;
@@ -427,27 +434,28 @@ EVENT_SINK_INVOKE(MailItemEvents)
                 }
             }
 
-          if (m_mail->crypt_state () == Mail::WantsSendInline)
+          if (m_mail->cryptState () == Mail::WantsSendInline)
             {
-              if (!m_mail->has_crypted_or_empty_body())
+              if (!m_mail->hasCryptedOrEmptyBody_o ())
                 {
                   log_debug ("%s:%s: Message %p mail %p cancelling send - "
                              "not encrypted or not empty body detected.",
                              SRCNAME, __func__, m_object, m_mail);
-                  gpgol_bug (m_mail->get_window (),
+                  gpgol_bug (m_mail->getWindow (),
                              ERR_WANTS_SEND_INLINE_BODY);
-                  m_mail->set_crypt_state (Mail::NoCryptMail);
+                  m_mail->setCryptState (Mail::NoCryptMail);
                   *(parms->rgvarg[0].pboolVal) = VARIANT_TRUE;
                   break;
                 }
               log_debug ("%s:%s: Passing send event for no-mime message %p.",
                          SRCNAME, __func__, m_object);
+              WKSHelper::instance()->allow_notify (1000);
               break;
             }
 
-          if (m_mail->crypt_state () == Mail::WantsSendMIME)
+          if (m_mail->cryptState () == Mail::WantsSendMIME)
             {
-              if (!m_mail->has_crypted_or_empty_body())
+              if (!m_mail->hasCryptedOrEmptyBody_o ())
                 {
 /* The safety checks here trigger too often. Somehow for some
    users the body is not empty after the encryption but when
@@ -462,12 +470,12 @@ EVENT_SINK_INVOKE(MailItemEvents)
 */
 #define DISABLE_SAFTEY_CHECKS
 #ifndef DISABLE_SAFTEY_CHECKS
-                  gpgol_bug (m_mail->get_window (),
+                  gpgol_bug (m_mail->getWindow (),
                              ERR_WANTS_SEND_MIME_BODY);
                   log_debug ("%s:%s: Message %p mail %p cancelling send mime - "
                              "not encrypted or not empty body detected.",
                              SRCNAME, __func__, m_object, m_mail);
-                  m_mail->set_crypt_state (Mail::NoCryptMail);
+                  m_mail->setCryptState (Mail::NoCryptMail);
                   *(parms->rgvarg[0].pboolVal) = VARIANT_TRUE;
                   break;
 #else
@@ -527,8 +535,9 @@ EVENT_SINK_INVOKE(MailItemEvents)
                 {
                   break;
                 }
-              log_debug ("%s:%s: Passing send event for message %p.",
+              log_debug ("%s:%s: Passing send event for mime-encrypted message %p.",
                          SRCNAME, __func__, m_object);
+              WKSHelper::instance()->allow_notify (1000);
               break;
             }
           else
@@ -557,10 +566,17 @@ EVENT_SINK_INVOKE(MailItemEvents)
              break;
            }
 
-          if (m_mail->is_crypto_mail () && !m_mail->needs_save ())
+          if (m_mail->isAboutToBeMoved())
             {
-              Mail *last_mail = Mail::get_last_mail ();
-              if (Mail::is_valid_ptr (last_mail))
+              log_debug ("%s:%s: Mail is about to be moved. Passing write for %p",
+                         SRCNAME, __func__, m_mail);
+              break;
+            }
+
+          if (m_mail->isCryptoMail () && !m_mail->needsSave ())
+            {
+              Mail *last_mail = Mail::getLastMail ();
+              if (Mail::isValidPtr (last_mail))
                 {
                   /* We want to identify here if there was a mail created that
                      should receive the contents of this mail. For this we check
@@ -578,7 +594,7 @@ EVENT_SINK_INVOKE(MailItemEvents)
                      Similarly if we crash or Outlook is closed before we see this
                      revert. But as we immediately revert after the write this should
                      also not happen. */
-                  const std::string lastSubject = last_mail->get_subject ();
+                  const std::string lastSubject = last_mail->getSubject_o ();
                   char *lastEntryID = get_oom_string (last_mail->item (), "EntryID");
                   int lastSize = get_oom_int (last_mail->item (), "Size");
                   std::string lastEntryStr;
@@ -596,7 +612,7 @@ EVENT_SINK_INVOKE(MailItemEvents)
 
                       /* This might be a forward. So don't invalidate yet. */
 
-                      // Mail::invalidate_last_mail ();
+                      // Mail::clearLastMail ();
 
                       do_in_ui_thread_async (REVERT_MAIL, m_mail);
                       return S_OK;
@@ -614,8 +630,8 @@ EVENT_SINK_INVOKE(MailItemEvents)
               return S_OK;
             }
 
-          if (m_mail->is_crypto_mail () && m_mail->needs_save () &&
-              m_mail->revert ())
+          if (m_mail->isCryptoMail () && m_mail->needsSave () &&
+              m_mail->revert_o ())
             {
               /* An error cleaning the mail should not happen normally.
                  But just in case there is an error we cancel the
@@ -625,8 +641,8 @@ EVENT_SINK_INVOKE(MailItemEvents)
               *(parms->rgvarg[0].pboolVal) = VARIANT_TRUE;
             }
 
-          if (!m_mail->is_crypto_mail () && m_mail->is_forwarded_crypto_mail () &&
-              !m_mail->needs_crypto () && m_mail->crypt_state () == Mail::NoCryptMail)
+          if (!m_mail->isCryptoMail () && m_mail->is_forwarded_crypto_mail () &&
+              !m_mail->needs_crypto_m () && m_mail->cryptState () == Mail::NoCryptMail)
             {
               /* We are sure now that while this is a forward of an encrypted
                * mail that the forward should not be signed or encrypted. So
@@ -635,7 +651,7 @@ EVENT_SINK_INVOKE(MailItemEvents)
               log_debug ("%s:%s: Writing unencrypted forward of crypt mail. "
                          "Removing attachments. mail: %p item: %p",
                          SRCNAME, __func__, m_mail, m_object);
-              if (m_mail->remove_our_attachments ())
+              if (m_mail->removeOurAttachments_o ())
                 {
                   // Worst case we forward some encrypted data here not
                   // a security problem, so let it pass.
@@ -643,34 +659,34 @@ EVENT_SINK_INVOKE(MailItemEvents)
                              SRCNAME, __func__);
                 }
               /* Remove marker because we did this now. */
-              m_mail->set_is_forwarded_crypto_mail (false);
+              m_mail->setIsForwardedCryptoMail (false);
             }
 
           log_debug ("%s:%s: Passing write event.",
                      SRCNAME, __func__);
-          m_mail->set_needs_save (false);
+          m_mail->setNeedsSave (false);
           break;
         }
       case AfterWrite:
         {
           log_oom_extra ("%s:%s: AfterWrite : %p",
                          SRCNAME, __func__, m_mail);
-          if (m_mail->crypt_state () == Mail::NeedsFirstAfterWrite)
+          if (m_mail->cryptState () == Mail::NeedsFirstAfterWrite)
             {
               /* Seen the first after write. Advance the state */
-              m_mail->set_crypt_state (Mail::NeedsActualCrypt);
-              if (m_mail->encrypt_sign_start ())
+              m_mail->setCryptState (Mail::NeedsActualCrypt);
+              if (m_mail->encryptSignStart_o ())
                 {
                   log_debug ("%s:%s: Encrypt sign start failes.",
                              SRCNAME, __func__);
-                  m_mail->set_crypt_state (Mail::NoCryptMail);
+                  m_mail->setCryptState (Mail::NoCryptMail);
                 }
               return S_OK;
             }
-          if (m_mail->crypt_state () == Mail::NeedsSecondAfterWrite)
+          if (m_mail->cryptState () == Mail::NeedsSecondAfterWrite)
             {
-              m_mail->set_crypt_state (Mail::NeedsUpdateInMAPI);
-              m_mail->update_crypt_mapi ();
+              m_mail->setCryptState (Mail::NeedsUpdateInMAPI);
+              m_mail->updateCryptMAPI_m ();
               return S_OK;
             }
           break;
@@ -679,7 +695,7 @@ EVENT_SINK_INVOKE(MailItemEvents)
         {
           log_oom_extra ("%s:%s: Close : %p",
                          SRCNAME, __func__, m_mail);
-          if (m_mail->is_crypto_mail ())
+          if (m_mail->isCryptoMail ())
             {
               /* Close. This happens when an Opened mail is closed.
                  To prevent the question of wether or not to save the changes
@@ -696,10 +712,10 @@ EVENT_SINK_INVOKE(MailItemEvents)
                              SRCNAME, __func__);
                   break;
                 }
-              if (m_mail->get_close_triggered ())
+              if (m_mail->getCloseTriggered ())
                 {
                   /* Our close with discard changes, pass through */
-                  m_mail->set_close_triggered (false);
+                  m_mail->setCloseTriggered (false);
                   return S_OK;
                 }
               *(parms->rgvarg[0].pboolVal) = VARIANT_TRUE;
@@ -722,6 +738,7 @@ EVENT_SINK_INVOKE(MailItemEvents)
           delete m_mail;
           log_oom_extra ("%s:%s: deletion done",
                          SRCNAME, __func__);
+          memdbg_dump ();
           return S_OK;
         }
       /* Fallthrough */
@@ -734,17 +751,62 @@ EVENT_SINK_INVOKE(MailItemEvents)
         {
           log_oom_extra ("%s:%s: %s : %p",
                          SRCNAME, __func__, is_reply ? "reply" : "forward", m_mail);
-          if (!m_mail->is_crypto_mail ())
+          int draft_flags = 0;
+          if (opt.encrypt_default)
             {
-              /* Non crypto mails do not interest us.*/
+              draft_flags = 1;
+            }
+          if (opt.sign_default)
+            {
+              draft_flags += 2;
+            }
+          bool is_crypto_mail = m_mail->isCryptoMail ();
+
+          /* If it is a crypto mail and the settings should not be taken
+           * from the crypto mail and always encrypt / sign is on. Or
+           * If it is not a crypto mail and we have automaticalls sign_encrypt. */
+          if ((is_crypto_mail && !opt.reply_crypt && draft_flags) ||
+              (!is_crypto_mail && draft_flags))
+            {
+              /* Check if we can use the dispval */
+                if (parms->cArgs == 2 && parms->rgvarg[1].vt == (VT_DISPATCH) &&
+                    parms->rgvarg[0].vt == (VT_BOOL | VT_BYREF))
+                {
+                  LPMESSAGE msg = get_oom_base_message (parms->rgvarg[1].pdispVal);
+                  if (msg)
+                    {
+                      set_gpgol_draft_info_flags (msg, draft_flags);
+                      gpgol_release (msg);
+                    }
+                  else
+                    {
+                      log_error ("%s:%s: Failed to get base message.",
+                                 SRCNAME, __func__);
+                    }
+                }
+              else
+                {
+                  log_error ("%s:%s: Unexpected parameters.",
+                             SRCNAME, __func__);
+                }
+            }
+
+          if (!is_crypto_mail)
+            {
+              /* Replys to non crypto mails do not interest us anymore. */
               break;
             }
-          Mail *last_mail = Mail::get_last_mail ();
-          if (Mail::is_valid_ptr (last_mail))
+
+          Mail *last_mail = Mail::getLastMail ();
+          if (Mail::isValidPtr (last_mail))
             {
               /* We want to identify here if there was a mail created that
                  should receive the contents of this mail. For this we check
                  for a forward in the same loop as a mail creation.
+
+                 We need to do it this complicated and can't just use
+                 get_mail_for_item because the mailitem pointer we get here
+                 is a different one then the one with which the mail was loaded.
               */
               char *lastEntryID = get_oom_string (last_mail->item (), "EntryID");
               int lastSize = get_oom_int (last_mail->item (), "Size");
@@ -764,7 +826,7 @@ EVENT_SINK_INVOKE(MailItemEvents)
                                  SRCNAME, __func__, last_mail,
                                  last_mail->item ());
 
-                      last_mail->set_is_forwarded_crypto_mail (true);
+                      last_mail->setIsForwardedCryptoMail (true);
                     }
                   else
                     {
@@ -773,7 +835,7 @@ EVENT_SINK_INVOKE(MailItemEvents)
                                  SRCNAME, __func__, last_mail,
                                  last_mail->item ());
                     }
-                  if (m_mail->is_block_html())
+                  if (m_mail->isBlockHTML ())
                     {
                       std::string caption = _("GpgOL") + std::string (": ");
                       caption += is_reply ? _("Dangerous reply") :
@@ -800,7 +862,7 @@ EVENT_SINK_INVOKE(MailItemEvents)
                     }
                 }
               // We can now invalidate the last mail
-              Mail::invalidate_last_mail ();
+              Mail::clearLastMail ();
             }
 
           log_oom_extra ("%s:%s: Reply Forward ReplyAll: %p",
@@ -810,7 +872,7 @@ EVENT_SINK_INVOKE(MailItemEvents)
               break;
             }
           int crypto_flags = 0;
-          if (!(crypto_flags = m_mail->get_crypto_flags ()))
+          if (!(crypto_flags = m_mail->getCryptoFlags ()))
             {
               break;
             }
@@ -838,8 +900,8 @@ EVENT_SINK_INVOKE(MailItemEvents)
         {
           log_oom_extra ("%s:%s: AttachmentRemove: %p",
                          SRCNAME, __func__, m_mail);
-          if (!m_mail->is_crypto_mail () || attachRemoveWarnShown ||
-              m_mail->attachment_remove_warning_disabled ())
+          if (!m_mail->isCryptoMail () || attachRemoveWarnShown ||
+              m_mail->attachmentRemoveWarningDisabled ())
             {
               return S_OK;
             }
