@@ -59,30 +59,7 @@ initialize_main (void)
   return log_mutex? 0 : -1;
 }
 
-/* Return nbytes of cryptographic strong random.  Caller needs to free
-   the returned buffer.  */
-static char *
-get_crypt_random (size_t nbytes)
-{
-  HCRYPTPROV prov;
-  char *buffer;
-
-  if (!CryptAcquireContext (&prov, NULL, NULL, PROV_RSA_FULL,
-                            (CRYPT_VERIFYCONTEXT|CRYPT_SILENT)) )
-    return NULL;
-
-  buffer = xmalloc (nbytes);
-  if (!CryptGenRandom (prov, nbytes, buffer))
-    {
-      xfree (buffer);
-      buffer = NULL;
-    }
-  CryptReleaseContext (prov, 0);
-  return buffer;
-}
-
-
-static void
+void
 i18n_init (void)
 {
   char *locale_dir;
@@ -111,16 +88,19 @@ get_gpgme_w32_inst_dir (void)
   char *gpg4win_dir = get_gpg4win_dir ();
   char *tmp;
   gpgrt_asprintf (&tmp, "%s\\bin\\gpgme-w32spawn.exe", gpg4win_dir);
+  memdbg_alloc (tmp);
 
   if (!access(tmp, R_OK))
     {
       xfree (tmp);
       gpgrt_asprintf (&tmp, "%s\\bin", gpg4win_dir);
+      memdbg_alloc (tmp);
       xfree (gpg4win_dir);
       return tmp;
     }
   xfree (tmp);
   gpgrt_asprintf (&tmp, "%s\\gpgme-w32spawn.exe", gpg4win_dir);
+  memdbg_alloc (tmp);
 
   if (!access(tmp, R_OK))
     {
@@ -158,7 +138,6 @@ DllMain (HINSTANCE hinst, DWORD reason, LPVOID reserved)
       /* Early initializations of our subsystems. */
       if (initialize_main ())
         return FALSE;
-      i18n_init ();
     }
   else if (reason == DLL_PROCESS_DETACH)
     {
@@ -168,14 +147,6 @@ DllMain (HINSTANCE hinst, DWORD reason, LPVOID reserved)
   return TRUE;
 }
 
-/* Return a new allocated IV of size NBYTES.  Caller must free it.  On
-   error NULL is returned. */
-void *
-create_initialization_vector (size_t nbytes)
-{
-  return get_crypt_random (nbytes);
-}
-
 static char *
 get_locale_dir (void)
 {
@@ -189,10 +160,10 @@ get_locale_dir (void)
 
   /* Build the key: "<instdir>/share/locale".  */
 #define SLDIR "\\share\\locale"
-  dname = malloc (strlen (instdir) + strlen (SLDIR) + 1);
+  dname = xmalloc (strlen (instdir) + strlen (SLDIR) + 1);
   if (!dname)
     {
-      free (instdir);
+      xfree (instdir);
       return NULL;
     }
   p = dname;
@@ -200,7 +171,7 @@ get_locale_dir (void)
   p += strlen (instdir);
   strcpy (p, SLDIR);
 
-  free (instdir);
+  xfree (instdir);
 
   return dname;
 }
@@ -209,15 +180,25 @@ get_locale_dir (void)
 static void
 drop_locale_dir (char *locale_dir)
 {
-  free (locale_dir);
+  xfree (locale_dir);
 }
 
 
+static int
+get_conf_bool (const char *name, int defaultVal)
+{
+  char *val = NULL;
+  int ret;
+  load_extension_value (name, &val);
+  ret = val == NULL ? defaultVal : *val != '1' ? 0 : 1;
+  xfree (val);
+  return ret;
+}
+
 /* Read option settings from the Registry. */
 void
 read_options (void)
 {
-  static int warnings_shown;
   char *val = NULL;
 
   /* Set the log file first so that output from this function is
@@ -275,7 +256,9 @@ read_options (void)
          after using the configuration dialog.  */
       store_extension_value ("enableDebug", "0");
     }
-  xfree (val); val = NULL;
+  /* Yes we use free here because memtracing did not track the alloc
+     as the option for debuging was not read before. */
+  free (val); val = NULL;
   if (opt.enable_debug)
     log_debug ("enabled debug flags:%s%s%s%s%s%s%s%s%s%s\n",
                (opt.enable_debug & DBG_IOWORKER)? " ioworker":"",
@@ -290,89 +273,27 @@ read_options (void)
                (opt.enable_debug & DBG_OOM_EXTRA)? " oom-extra":""
                );
 
+  opt.enable_smime = get_conf_bool ("enableSmime", 0);
+  opt.encrypt_default = get_conf_bool ("encryptDefault", 0);
+  opt.sign_default = get_conf_bool ("signDefault", 0);
+  opt.inline_pgp = get_conf_bool ("inlinePGP", 0);
+  opt.reply_crypt = get_conf_bool ("replyCrypt", 1);
+  opt.prefer_smime = get_conf_bool ("preferSmime", 0);
+  opt.autoresolve = get_conf_bool ("autoresolve", 1);
+  opt.automation = get_conf_bool ("automation", 1);
+  opt.autosecure = get_conf_bool ("autosecure", 1);
+  opt.autotrust = get_conf_bool ("autotrust", 0);
+  opt.sync_enc = get_conf_bool ("syncEnc", 0);
+  opt.smime_html_warn_shown = get_conf_bool ("smimeHtmlWarnShown", 0);
 
-  load_extension_value ("enableSmime", &val);
-  opt.enable_smime = !val ? 0 : atoi (val);
-  xfree (val); val = NULL;
-
-  load_extension_value ("encryptDefault", &val);
-  opt.encrypt_default = val == NULL || *val != '1'? 0 : 1;
-  xfree (val); val = NULL;
-
-  load_extension_value ("signDefault", &val);
-  opt.sign_default = val == NULL || *val != '1'? 0 : 1;
-  xfree (val); val = NULL;
-
-  load_extension_value ("defaultKey", &val);
-  set_default_key (val);
-  xfree (val); val = NULL;
-
-  load_extension_value ("gitCommit", &val);
-  opt.git_commit = val? strtoul (val, NULL, 16) : 0;
-  xfree (val); val = NULL;
-
-  load_extension_value ("formsRevision", &val);
-  opt.forms_revision = val? atol (val) : 0;
-  xfree (val); val = NULL;
-
-  load_extension_value ("announceNumber", &val);
-  opt.announce_number = val? atol (val) : 0;
-  xfree (val); val = NULL;
-
-  load_extension_value ("inlinePGP", &val);
-  opt.inline_pgp = val == NULL || *val != '1'? 0 : 1;
-  xfree (val); val = NULL;
-  load_extension_value ("autoresolve", &val);
-  opt.autoresolve = val == NULL ? 1 : *val != '1' ? 0 : 1;
-  xfree (val); val = NULL;
-  load_extension_value ("replyCrypt", &val);
-  opt.reply_crypt = val == NULL ? 1 : *val != '1' ? 0 : 1;
-  xfree (val); val = NULL;
-  load_extension_value ("smimeHtmlWarnShown", &val);
-  opt.smime_html_warn_shown = val == NULL || *val != '1'? 0 : 1;
-  xfree (val); val = NULL;
-  /* Note, that on purpose these flags are only Registry changeable.
-     The format of the entry is a string of of "0" and "1" digits; see
-     the switch below for a description. */
-  memset (&opt.compat, 0, sizeof opt.compat);
-  load_extension_value ("compatFlags", &val);
-  if (val)
+  if (!opt.automation)
     {
-      const char *s = val;
-      int i, x;
-
-      for (s=val, i=0; *s; s++, i++)
-        {
-          x = *s == '1';
-          switch (i)
-            {
-            case 0: opt.compat.no_msgcache = x; break;
-            case 1: opt.compat.no_pgpmime = x; break;
-            case 2: opt.compat.no_oom_write = x; break;
-            case 3: opt.compat.no_preview_info = x; break;
-            case 4: opt.compat.old_reply_hack = x; break;
-            case 5: opt.compat.auto_decrypt = x; break;
-            case 6: opt.compat.no_attestation = x; break;
-            case 7: opt.compat.use_mwfmo = x; break;
-            }
-        }
-      log_debug ("Note: using compatibility flags: %s", val);
+      // Disabling automation is a shorthand to disable the
+      // others, too.
+      opt.autosecure = 0;
+      opt.autoresolve = 0;
+      opt.autotrust = 0;
     }
-
-  if (!warnings_shown)
-    {
-      char tmpbuf[512];
-
-      warnings_shown = 1;
-      if (val && *val)
-        {
-          snprintf (tmpbuf, sizeof tmpbuf,
-                    _("Note: Using compatibility flags: %s"), val);
-          MessageBox (NULL, tmpbuf, _("GpgOL"), MB_ICONWARNING|MB_OK);
-        }
-    }
-  xfree (val); val = NULL;
-
 }
 
 
@@ -387,16 +308,6 @@ write_options (void)
     int  value;
     char *s_val;
   } table[] = {
-    {"enableSmime",              0, opt.enable_smime, NULL},
-    {"encryptDefault",           0, opt.encrypt_default, NULL},
-    {"signDefault",              0, opt.sign_default, NULL},
-    {"logFile",                  2, 0, (char*) get_log_file ()},
-    {"gitCommit",                4, opt.git_commit, NULL},
-    {"formsRevision",            1, opt.forms_revision, NULL},
-    {"announceNumber",           1, opt.announce_number, NULL},
-    {"inlinePGP",                0, opt.inline_pgp, NULL},
-    {"autoresolve",              0, opt.autoresolve, NULL},
-    {"replyCrypt",               0, opt.reply_crypt, NULL},
     {"smimeHtmlWarnShown",       0, opt.smime_html_warn_shown, NULL},
     {NULL, 0, 0, NULL}
   };
