@@ -195,7 +195,7 @@ t2body (MimeDataProvider *provider, rfc822parse_t msg)
           ctx->is_base64_encoded = 1;
           b64_init (&ctx->base64);
         }
-      free (p);
+      xfree (p);
     }
 
   /* Get the filename from the header.  */
@@ -489,6 +489,7 @@ MimeDataProvider::MimeDataProvider(bool no_headers) :
   m_has_html_body(false),
   m_collect_everything(no_headers)
 {
+  memdbg_ctor ("MimeDataProvider");
   m_mime_ctx = (mime_context_t) xcalloc (1, sizeof *m_mime_ctx);
   m_mime_ctx->msg = rfc822parse_open (message_cb, this);
   m_mime_ctx->mimestruct_tail = &m_mime_ctx->mimestruct;
@@ -501,6 +502,7 @@ MimeDataProvider::MimeDataProvider(LPSTREAM stream, bool no_headers):
   if (stream)
     {
       stream->AddRef ();
+      memdbg_addRef (stream);
     }
   else
     {
@@ -524,6 +526,7 @@ MimeDataProvider::MimeDataProvider(FILE *stream, bool no_headers):
 
 MimeDataProvider::~MimeDataProvider()
 {
+  memdbg_dtor ("MimeDataProvider");
   log_debug ("%s:%s", SRCNAME, __func__);
   while (m_mime_ctx->mimestruct)
     {
@@ -558,11 +561,21 @@ MimeDataProvider::read(void *buffer, size_t size)
   log_mime_parser ("%s:%s: Reading: " SIZE_T_FORMAT "Bytes",
                  SRCNAME, __func__, size);
   ssize_t bRead = m_crypto_data.read (buffer, size);
-  if (opt.enable_debug & DBG_MIME_PARSER)
+  if (opt.enable_debug & DBG_MIME_DATA && bRead)
     {
       std::string buf ((char *)buffer, bRead);
-      log_mime_parser ("%s:%s: Data: \"%s\"",
-                     SRCNAME, __func__, buf.c_str());
+
+      if (!is_binary (buf))
+        {
+          log_mime_data ("%s:%s: Data: \n------\n%s\n------",
+                           SRCNAME, __func__, buf.c_str());
+        }
+      else
+        {
+          log_mime_data ("%s:%s: Hex Data: \n------\n%s\n------",
+                           SRCNAME, __func__,
+                           string_to_hex (buf).c_str ());
+        }
     }
   return bRead;
 }
@@ -601,8 +614,8 @@ MimeDataProvider::collect_input_lines(const char *input, size_t insize)
               pos--;
             }
 
-          log_mime_parser("%s:%s: Parsing line=`%.*s'\n",
-                          SRCNAME, __func__, (int)pos, linebuf);
+          log_mime_data ("%s:%s: Parsing line=`%.*s'\n",
+                         SRCNAME, __func__, (int)pos, linebuf);
           /* Check the next state */
           if (rfc822parse_insert (m_mime_ctx->msg,
                                   (unsigned char*) linebuf,
@@ -640,7 +653,7 @@ MimeDataProvider::collect_input_lines(const char *input, size_t insize)
                 {
                   m_crypto_data.write ("\r\n", 2);
                 }
-              log_mime_parser ("Writing raw crypto data: %.*s",
+              log_mime_data ("Writing raw crypto data: %.*s",
                                (int)pos, linebuf);
               m_crypto_data.write (linebuf, pos);
               m_mime_ctx->collect_crypto_data = 2;
@@ -1017,7 +1030,7 @@ MimeDataProvider::create_attachment()
   /* TODO handle encoding */
 }
 
-const std::string &MimeDataProvider::get_body()
+const std::string &MimeDataProvider::get_body ()
 {
   if (m_rawbuf.size())
     {
@@ -1031,7 +1044,7 @@ const std::string &MimeDataProvider::get_body()
   return m_body;
 }
 
-const std::string &MimeDataProvider::get_html_body()
+const std::string &MimeDataProvider::get_html_body ()
 {
   if (!m_has_html_body)
     {
