@@ -97,7 +97,7 @@ int
 sink_string_write (sink_t sink, const void *data, size_t datalen)
 {
   Mail *mail = static_cast<Mail *>(sink->cb_data);
-  mail->append_to_inline_body (std::string((char*)data, datalen));
+  mail->appendToInlineBody (std::string((char*)data, datalen));
   return 0;
 }
 
@@ -143,6 +143,7 @@ create_mapi_attachment (LPMESSAGE message, sink_t sink,
   sink->cb_data = NULL;
   sink->writefnc = NULL;
   hr = message->CreateAttach(NULL, 0, &pos, &att);
+  memdbg_addRef (att);
   if (hr)
     {
       log_error ("%s:%s: can't create attachment: hr=%#lx\n",
@@ -175,7 +176,7 @@ create_mapi_attachment (LPMESSAGE message, sink_t sink,
 
   /* We better insert a short filename. */
   prop.ulPropTag = PR_ATTACH_FILENAME_A;
-  prop.Value.lpszA = strdup (MIMEATTACHFILENAME);
+  prop.Value.lpszA = xstrdup (MIMEATTACHFILENAME);
   hr = HrSetOneProp ((LPMAPIPROP)att, &prop);
   xfree (prop.Value.lpszA);
   if (hr)
@@ -196,8 +197,8 @@ create_mapi_attachment (LPMESSAGE message, sink_t sink,
   if (!hr)
     {
       prop.ulPropTag = PR_ATTACH_MIME_TAG_A;
-      prop.Value.lpszA = overrideMimeTag ? strdup (overrideMimeTag) :
-                         strdup ("multipart/signed");
+      prop.Value.lpszA = overrideMimeTag ? xstrdup (overrideMimeTag) :
+                         xstrdup ("multipart/signed");
       if (overrideMimeTag)
         {
           log_debug ("%s:%s: using override mimetag: %s\n",
@@ -214,7 +215,7 @@ create_mapi_attachment (LPMESSAGE message, sink_t sink,
     }
 
   punk = NULL;
-  hr = att->OpenProperty(PR_ATTACH_DATA_BIN, &IID_IStream, 0,
+  hr = gpgol_openProperty (att, PR_ATTACH_DATA_BIN, &IID_IStream, 0,
                          (MAPI_CREATE|MAPI_MODIFY), &punk);
   if (FAILED (hr))
     {
@@ -679,10 +680,19 @@ infer_content_type (const char * /*data*/, size_t /*datalen*/,
 
   *force_b64 = 0;
   if (filename)
-    suffix = strrchr (filename, '.');
-
-  if (!suffix.empty())
     {
+      const char *dot = strrchr (filename, '.');
+
+      if (dot)
+        {
+          suffix = dot;
+        }
+    }
+
+  /* Check for at least one char after the dot. */
+  if (suffix.size() > 1)
+    {
+      /* Erase the dot */
       suffix.erase(0, 1);
       std::transform(suffix.begin(), suffix.end(), suffix.begin(), ::tolower);
       for (i=0; suffix_table[i].suffix; i++)
@@ -1017,7 +1027,7 @@ write_attachments (sink_t sink,
 static int
 is_related (Mail *mail, mapi_attach_item_t *table)
 {
-  if (!mail || !mail->is_html_alternative () || !table)
+  if (!mail || !mail->isHTMLAlternative () || !table)
     {
       return 0;
     }
@@ -1140,11 +1150,11 @@ finalize_message (LPMESSAGE message, mapi_attach_item_t *att_table,
   prop.ulPropTag = PR_MESSAGE_CLASS_A;
   if (encrypt)
     {
-      prop.Value.lpszA = strdup ("IPM.Note.InfoPathForm.GpgOL.SMIME.MultipartSigned");
+      prop.Value.lpszA = xstrdup ("IPM.Note.InfoPathForm.GpgOL.SMIME.MultipartSigned");
     }
   else
     {
-      prop.Value.lpszA = strdup ("IPM.Note.InfoPathForm.GpgOLS.SMIME.MultipartSigned");
+      prop.Value.lpszA = xstrdup ("IPM.Note.InfoPathForm.GpgOLS.SMIME.MultipartSigned");
     }
 
   if (!is_inline)
@@ -1278,7 +1288,7 @@ add_body (Mail *mail, const char *boundary, sink_t sink,
   bool is_alternative = false;
   if (mail)
     {
-      is_alternative = mail->is_html_alternative ();
+      is_alternative = mail->isHTMLAlternative ();
     }
 
   int rc = 0;
@@ -1325,7 +1335,7 @@ add_body (Mail *mail, const char *boundary, sink_t sink,
   /* Now the html body. It is somehow not accessible through PR_HTML,
      OutlookSpy also shows MAPI Unsupported (but shows the data) strange.
      We just cache it. Memory is cheap :-) */
-  char *html_body = mail->take_cached_html_body();
+  char *html_body = mail->takeCachedHTMLBody ();
   if (!html_body)
     {
       log_error ("%s:%s: BUG: Body but no html body in alternative mail?",
