@@ -29,6 +29,9 @@ DEFINE_GUID (IID_IMultiLanguage, 0x275c23e1,0x3747,0x11d0,0x9f,
 #include <mlang.h>
 #undef INITGUID
 
+#include "dialogs.h"
+#include "dispcache.h"
+
 #include "mlang-charset.h"
 
 char *ansi_charset_to_utf8 (const char *charset, const char *input,
@@ -48,11 +51,24 @@ char *ansi_charset_to_utf8 (const char *charset, const char *input,
     {
       log_debug ("%s:%s: No charset / codepage returning plain.",
                  SRCNAME, __func__);
-      return strdup (input);
+      return xstrdup (input);
+    }
+
+  auto cache = DispCache::instance ();
+  LPDISPATCH cachedLang = cache->getDisp (DISPID_MLANG_CHARSET);
+
+  if (!cachedLang)
+    {
+      CoCreateInstance(CLSID_CMultiLanguage, NULL, CLSCTX_INPROC_SERVER,
+                       IID_IMultiLanguage, (void**)&multilang);
+      memdbg_addRef (multilang);
+      cache->addDisp (DISPID_MLANG_CHARSET, (LPDISPATCH) multilang);
+    }
+  else
+    {
+      multilang = (LPMULTILANGUAGE) cachedLang;
     }
 
-  CoCreateInstance(CLSID_CMultiLanguage, NULL, CLSCTX_INPROC_SERVER,
-                   IID_IMultiLanguage, (void**)&multilang);
 
   if (!multilang)
     {
@@ -83,7 +99,7 @@ char *ansi_charset_to_utf8 (const char *charset, const char *input,
           log_error ("%s:%s: Failed to find charset for: %s",
                      SRCNAME, __func__, charset);
           gpgol_release (multilang);
-          return strdup(input);
+          return xstrdup (input);
         }
       enc = (mime_info.uiInternetEncoding == 0) ? mime_info.uiCodePage :
                                                   mime_info.uiInternetEncoding;
@@ -107,7 +123,6 @@ char *ansi_charset_to_utf8 (const char *charset, const char *input,
 
   err = multilang->ConvertStringToUnicode(&mode, enc, const_cast<char*>(input),
                                           &uinlen, buf, &wlen);
-  gpgol_release (multilang);
   if (FAILED (err))
     {
       log_error ("%s:%s: Failed conversion 2.",
