@@ -33,7 +33,33 @@
 #include "oomhelp.h"
 #include "gpgoladdin.h"
 
+HRESULT
+gpgol_queryInterface (LPUNKNOWN pObj, REFIID riid, LPVOID FAR *ppvObj)
+{
+  HRESULT ret = pObj->QueryInterface (riid, ppvObj);
+  if ((opt.enable_debug & DBG_OOM_EXTRA) && *ppvObj)
+    {
+      memdbg_addRef (*ppvObj);
+    }
+  return ret;
+}
 
+HRESULT
+gpgol_openProperty (LPMAPIPROP obj, ULONG ulPropTag, LPCIID lpiid,
+                    ULONG ulInterfaceOptions, ULONG ulFlags,
+                    LPUNKNOWN FAR * lppUnk)
+{
+  HRESULT ret = obj->OpenProperty (ulPropTag, lpiid,
+                                   ulInterfaceOptions, ulFlags,
+                                   lppUnk);
+  if ((opt.enable_debug & DBG_OOM_EXTRA) && *lppUnk)
+    {
+      memdbg_addRef (*lppUnk);
+      log_debug ("%s:%s: OpenProperty on %p prop %lx result %p",
+                 SRCNAME, __func__, obj,  ulPropTag, *lppUnk);
+    }
+  return ret;
+}
 /* Return a malloced string with the utf-8 encoded name of the object
    or NULL if not available.  */
 char *
@@ -48,6 +74,7 @@ get_object_name (LPUNKNOWN obj)
   if (!obj)
     goto leave;
 
+  /* We can't use gpgol_queryInterface here to avoid recursion */
   obj->QueryInterface (IID_IDispatch, (void **)&disp);
   if (!disp)
     goto leave;
@@ -73,9 +100,9 @@ get_object_name (LPUNKNOWN obj)
 
  leave:
   if (tinfo)
-    gpgol_release (tinfo);
+    tinfo->Release ();
   if (disp)
-    gpgol_release (disp);
+    disp->Release ();
 
   return name;
 }
@@ -188,7 +215,7 @@ get_oom_object (LPDISPATCH pStart, const char *fullname)
           gpgol_release (pDisp);
           pDisp = NULL;
         }
-      if (pObj->QueryInterface (IID_IDispatch, (LPVOID*)&pDisp) != S_OK)
+      if (gpgol_queryInterface (pObj, IID_IDispatch, (LPVOID*)&pDisp) != S_OK)
         {
           log_error ("%s:%s Object does not support IDispatch",
                      SRCNAME, __func__);
@@ -203,7 +230,13 @@ get_oom_object (LPDISPATCH pStart, const char *fullname)
         return NULL;  /* The object has no IDispatch interface.  */
       if (!*fullname)
         {
-          log_oom ("%s:%s:         got %p",SRCNAME, __func__, pDisp);
+          if ((opt.enable_debug & DBG_OOM))
+            {
+              pDisp->AddRef ();
+              int ref = pDisp->Release ();
+              log_oom ("%s:%s:         got %p with %i refs",
+                       SRCNAME, __func__, pDisp, ref);
+            }
           return pDisp; /* Ready.  */
         }
       
@@ -316,10 +349,10 @@ get_oom_object (LPDISPATCH pStart, const char *fullname)
         SysFreeString (parmstr);
       if (hr != S_OK || vtResult.vt != VT_DISPATCH)
         {
-          log_debug ("%s:%s: failure: '%s' p=%p vt=%d hr=0x%x argErr=0x%x",
+          log_debug ("%s:%s: failure: '%s' p=%p vt=%d hr=0x%x argErr=0x%x dispid=0x%x",
                      SRCNAME, __func__,
                      name, vtResult.pdispVal, vtResult.vt, (unsigned int)hr,
-                     (unsigned int)argErr);
+                     (unsigned int)argErr, (unsigned int)dispid);
           dump_excepinfo (execpinfo);
           VariantClear (&vtResult);
           gpgol_release (pDisp);
@@ -327,7 +360,9 @@ get_oom_object (LPDISPATCH pStart, const char *fullname)
         }
 
       pObj = vtResult.pdispVal;
+      memdbg_addRef (pObj);
     }
+  gpgol_release (pDisp);
   log_debug ("%s:%s: no object", SRCNAME, __func__);
   return NULL;
 }
@@ -694,7 +729,10 @@ get_oom_iunknown (LPDISPATCH pDisp, const char *name)
         log_debug ("%s:%s: Property `%s' is not of class IUnknown (vt=%d)",
                    SRCNAME, __func__, name, rVariant.vt);
       else
-        return rVariant.punkVal;
+        {
+          memdbg_addRef (rVariant.punkVal);
+          return rVariant.punkVal;
+        }
 
       VariantClear (&rVariant);
     }
@@ -753,7 +791,8 @@ get_oom_control_bytag (LPDISPATCH pDisp, const char *tag)
   SysFreeString (bstring);
   if (hr == S_OK && rVariant.vt == VT_DISPATCH && rVariant.pdispVal)
     {
-      rVariant.pdispVal->QueryInterface (IID_IDispatch, (LPVOID*)&result);
+      gpgol_queryInterface (rVariant.pdispVal, IID_IDispatch,
+                            (LPVOID*)&result);
       gpgol_release (rVariant.pdispVal);
       if (!result)
         log_debug ("%s:%s: Object with tag `%s' has no dispatch intf.",
@@ -940,6 +979,7 @@ put_pa_string (LPDISPATCH pDisp, const char *dasl_id, const char *value)
 {
   wchar_t *w_value = utf8_to_wchar (value);
   BSTR b_value = SysAllocString(w_value);
+  xfree (w_value);
   VARIANT var;
   VariantInit (&var);
   var.vt = VT_BSTR;
@@ -1105,6 +1145,52 @@ get_pa_int (LPDISPATCH pDisp, const char *property, int *rInt)
   return 0;
 }
 
+/* Helper for exchange address lookup. */
+static char *
+get_recipient_addr_entry_fallbacks_ex (LPDISPATCH addr_entry)
+{
+  /* Maybe check for type here? We are pretty sure that we are exchange */
+
+  /* According to MSDN Message Boards the PR_EMS_AB_PROXY_ADDRESSES_DASL
+     is more avilable then the SMTP Address. */
+  char *ret = get_pa_string (addr_entry, PR_EMS_AB_PROXY_ADDRESSES_DASL);
+  if (ret)
+    {
+      log_debug ("%s:%s: Found recipient through AB_PROXY: %s",
+                 SRCNAME, __func__, ret);
+
+      char *smtpbegin = strstr(ret, "SMTP:");
+      if (smtpbegin == ret)
+        {
+          ret += 5;
+        }
+      return ret;
+    }
+  else
+    {
+      log_debug ("%s:%s: Failed AB_PROXY lookup.",
+                 SRCNAME, __func__);
+    }
+
+  LPDISPATCH ex_user = get_oom_object (addr_entry, "GetExchangeUser");
+  if (!ex_user)
+    {
+      log_debug ("%s:%s: Failed to find ExchangeUser",
+                 SRCNAME, __func__);
+      return nullptr;
+    }
+
+  ret = get_oom_string (ex_user, "PrimarySmtpAddress");
+  gpgol_release (ex_user);
+  if (ret)
+    {
+      log_debug ("%s:%s: Found recipient through exchange user primary smtp address: %s",
+                 SRCNAME, __func__, ret);
+      return ret;
+    }
+  return nullptr;
+}
+
 /* Helper for additional fallbacks in recipient lookup */
 static char *
 get_recipient_addr_fallbacks (LPDISPATCH recipient)
@@ -1122,68 +1208,147 @@ get_recipient_addr_fallbacks (LPDISPATCH recipient)
       return nullptr;
     }
 
-  /* Maybe check for type here? We are pretty sure that we are exchange */
+  char *ret = get_recipient_addr_entry_fallbacks_ex (addr_entry);
 
-  /* According to MSDN Message Boards the PR_EMS_AB_PROXY_ADDRESSES_DASL
-     is more avilable then the SMTP Address. */
-  char *ret = get_pa_string (addr_entry, PR_EMS_AB_PROXY_ADDRESSES_DASL);
-  if (ret)
-    {
-      log_debug ("%s:%s: Found recipient through AB_PROXY: %s",
-                 SRCNAME, __func__, ret);
-
-      char *smtpbegin = strstr(ret, "SMTP:");
-      if (smtpbegin == ret)
-        {
-          ret += 5;
-        }
-      gpgol_release (addr_entry);
-      return ret;
-    }
-  else
-    {
-      log_debug ("%s:%s: Failed AB_PROXY lookup.",
-                 SRCNAME, __func__);
-    }
-
-  LPDISPATCH ex_user = get_oom_object (addr_entry, "GetExchangeUser");
   gpgol_release (addr_entry);
-  if (!ex_user)
-    {
-      log_debug ("%s:%s: Failed to find ExchangeUser",
-                 SRCNAME, __func__);
-      return nullptr;
-    }
 
-  ret = get_oom_string (ex_user, "PrimarySmtpAddress");
-  gpgol_release (ex_user);
-  if (ret)
-    {
-      log_debug ("%s:%s: Found recipient through exchange user primary smtp address: %s",
-                 SRCNAME, __func__, ret);
-      return ret;
-    }
-
-  return nullptr;
+  return ret;
 }
 
-/* Gets a malloced NULL terminated array of recipent strings from
-   an OOM recipients Object. */
-char **
-get_oom_recipients (LPDISPATCH recipients)
+/* Try to resolve a recipient group and add it to the recipients vector.
+
+   returns true on success.
+*/
+static bool
+try_resolve_group (LPDISPATCH addrEntry, std::vector<std::string> &ret)
+{
+  /* Get the name for debugging */
+  std::string name;
+  char *cname = get_oom_string (addrEntry, "Name");
+  if (cname)
+    {
+      name = cname;
+    }
+  xfree (cname);
+
+  int type = get_oom_int (addrEntry, "AddressEntryUserType");
+
+  if (type != DISTRIBUTION_LIST_ADDRESS_ENTRY_TYPE)
+    {
+      log_mime_parser ("%s:%s: type of %s is %i",
+                       SRCNAME, __func__, name.c_str(), type);
+      return false;
+    }
+
+  LPDISPATCH members = get_oom_object (addrEntry, "Members");
+  addrEntry = nullptr;
+
+  if (!members)
+    {
+      TRACEPOINT;
+      return false;
+    }
+
+  int count = get_oom_int (members, "Count");
+
+  if (!count)
+    {
+      TRACEPOINT;
+      gpgol_release (members);
+      return false;
+    }
+
+  bool foundOne = false;
+  for (int i = 1; i <= count; i++)
+    {
+      auto item_str = std::string("Item(") + std::to_string (i) + ")";
+      LPDISPATCH entry = get_oom_object (members, item_str.c_str());
+      if (!entry)
+        {
+          TRACEPOINT;
+          continue;
+        }
+      std::string entryName;
+      char *entry_name = get_oom_string (entry, "Name");
+      if (entry_name)
+        {
+          entryName = entry_name;
+          xfree (entry_name);
+        }
+
+      int subType = get_oom_int (entry, "AddressEntryUserType");
+      /* Resolve recursively, yeah fun. */
+      if (subType == DISTRIBUTION_LIST_ADDRESS_ENTRY_TYPE)
+        {
+          log_debug ("%s:%s: recursive address entry %s",
+                     SRCNAME, __func__,
+                     (opt.enable_debug & DBG_MIME_PARSER) ?
+                     entryName.c_str() : "omitted");
+          if (try_resolve_group (entry, ret))
+            {
+              foundOne = true;
+              gpgol_release (entry);
+              continue;
+            }
+        }
+
+      /* Resolve directly ? */
+      char *addrtype = get_pa_string (entry, PR_ADDRTYPE_DASL);
+      if (addrtype && !strcmp (addrtype, "SMTP"))
+        {
+          xfree (addrtype);
+          char *resolved = get_pa_string (entry, PR_EMAIL_ADDRESS_DASL);
+          if (resolved)
+            {
+              ret.push_back(resolved);
+              foundOne = true;
+              gpgol_release (entry);
+              continue;
+            }
+        }
+      xfree (addrtype);
+
+      /* Resolve through Exchange API */
+      char *ex_resolved = get_recipient_addr_entry_fallbacks_ex (entry);
+      if (ex_resolved)
+        {
+          ret.push_back (ex_resolved);
+          foundOne = true;
+          gpgol_release (entry);
+          continue;
+        }
+
+      gpgol_release (entry);
+      log_debug ("%s:%s: failed to resolve name %s",
+                 SRCNAME, __func__,
+                 (opt.enable_debug & DBG_MIME_PARSER) ?
+                 entryName.c_str() : "omitted");
+    }
+  gpgol_release (members);
+  if (!foundOne)
+    {
+      log_debug ("%s:%s: failed to resolve group %s",
+                 SRCNAME, __func__,
+                 (opt.enable_debug & DBG_MIME_PARSER) ?
+                 name.c_str() : "omitted");
+    }
+  return foundOne;
+}
+
+/* Gets the resolved smtp addresses of the recpients. */
+std::vector<std::string>
+get_oom_recipients (LPDISPATCH recipients, bool *r_err)
 {
   int recipientsCnt = get_oom_int (recipients, "Count");
-  char **recipientAddrs = NULL;
+  std::vector<std::string> ret;
   int i;
 
   if (!recipientsCnt)
     {
-      return NULL;
+      return ret;
     }
 
   /* Get the recipients */
-  recipientAddrs = (char**) xmalloc((recipientsCnt + 1) * sizeof(char*));
-  recipientAddrs[recipientsCnt] = NULL;
   for (i = 1; i <= recipientsCnt; i++)
     {
       char buf[16];
@@ -1193,15 +1358,31 @@ get_oom_recipients (LPDISPATCH recipients)
       if (!recipient)
         {
           /* Should be impossible */
-          recipientAddrs[i-1] = NULL;
           log_error ("%s:%s: could not find Item %i;",
                      SRCNAME, __func__, i);
+          if (r_err)
+            {
+              *r_err = true;
+            }
           break;
         }
+
+      LPDISPATCH addrEntry = get_oom_object (recipient, "AddressEntry");
+      if (addrEntry && try_resolve_group (addrEntry, ret))
+        {
+          log_debug ("%s:%s: Resolved recipient group",
+                     SRCNAME, __func__);
+          gpgol_release (recipient);
+          gpgol_release (addrEntry);
+          continue;
+        }
+      gpgol_release (addrEntry);
+
       char *resolved = get_pa_string (recipient, PR_SMTP_ADDRESS_DASL);
       if (resolved)
         {
-          recipientAddrs[i-1] = resolved;
+          ret.push_back (resolved);
+          xfree (resolved);
           gpgol_release (recipient);
           continue;
         }
@@ -1209,7 +1390,8 @@ get_oom_recipients (LPDISPATCH recipients)
       resolved = get_recipient_addr_fallbacks (recipient);
       if (resolved)
         {
-          recipientAddrs[i-1] = resolved;
+          ret.push_back (resolved);
+          xfree (resolved);
           gpgol_release (recipient);
           continue;
         }
@@ -1218,9 +1400,18 @@ get_oom_recipients (LPDISPATCH recipients)
       gpgol_release (recipient);
       log_debug ("%s:%s: Failed to look up Address probably EX addr is returned",
                  SRCNAME, __func__);
-      recipientAddrs[i-1] = address;
+      if (address)
+        {
+          ret.push_back (address);
+          xfree (address);
+        }
+      else if (r_err)
+        {
+          *r_err = true;
+        }
     }
-  return recipientAddrs;
+
+  return ret;
 }
 
 /* Add an attachment to the outlook dispatcher disp
@@ -1309,7 +1500,7 @@ get_object_by_id (LPDISPATCH pDisp, REFIID id)
   if (!pDisp)
     return NULL;
 
-  if (pDisp->QueryInterface (id, (void **)&disp) != S_OK)
+  if (gpgol_queryInterface(pDisp, id, (void **)&disp) != S_OK)
     return NULL;
   return disp;
 }
@@ -1338,6 +1529,7 @@ get_strong_reference (LPDISPATCH mail)
       ret = var.pdispVal;
       log_oom ("%s:%s: Got strong ref %p for %p",
                SRCNAME, __func__, ret, mail);
+      memdbg_addRef (ret);
     }
   else
     {
@@ -1386,6 +1578,7 @@ get_oom_base_message_from_mapi (LPDISPATCH mapi_message)
   log_oom_extra("%s:%s: About to call GetBaseMessage.",
                 SRCNAME, __func__);
   hr = secureMessage->GetBaseMessage (&message);
+  memdbg_addRef (message);
   gpgol_release (secureMessage);
   if (hr != S_OK)
     {
@@ -1412,9 +1605,10 @@ get_oom_base_message (LPDISPATCH mailitem)
   return ret;
 }
 
-int
-invoke_oom_method_with_parms (LPDISPATCH pDisp, const char *name,
-                              VARIANT *rVariant, DISPPARAMS *params)
+static int
+invoke_oom_method_with_parms_type (LPDISPATCH pDisp, const char *name,
+                                   VARIANT *rVariant, DISPPARAMS *params,
+                                   int type)
 {
   HRESULT hr;
   DISPID dispid;
@@ -1427,7 +1621,7 @@ invoke_oom_method_with_parms (LPDISPATCH pDisp, const char *name,
       DISPPARAMS dispparams = {NULL, NULL, 0, 0};
 
       hr = pDisp->Invoke (dispid, IID_NULL, LOCALE_SYSTEM_DEFAULT,
-                          DISPATCH_METHOD, params ? params : &dispparams,
+                          type, params ? params : &dispparams,
                           rVariant, &execpinfo, NULL);
       if (hr != S_OK)
         {
@@ -1441,6 +1635,14 @@ invoke_oom_method_with_parms (LPDISPATCH pDisp, const char *name,
   return 0;
 }
 
+int
+invoke_oom_method_with_parms (LPDISPATCH pDisp, const char *name,
+                              VARIANT *rVariant, DISPPARAMS *params)
+{
+  return invoke_oom_method_with_parms_type (pDisp, name, rVariant, params,
+                                            DISPATCH_METHOD);
+}
+
 int
 invoke_oom_method (LPDISPATCH pDisp, const char *name, VARIANT *rVariant)
 {
@@ -1477,7 +1679,7 @@ get_oom_mapi_session ()
       return NULL;
     }
   session = NULL;
-  hr = mapiobj->QueryInterface (IID_IMAPISession, (void**)&session);
+  hr = gpgol_queryInterface (mapiobj, IID_IMAPISession, (void**)&session);
   gpgol_release (mapiobj);
   if (hr != S_OK || !session)
     {
@@ -1618,6 +1820,7 @@ ensure_category_exists (LPDISPATCH application, const char *category, int color)
           if (!category_obj)
             {
               TRACEPOINT;
+              gpgol_release (categories);
               break;
             }
           char *name = get_oom_string (category_obj, "Name");
@@ -1717,7 +1920,7 @@ generate_uid ()
   unsigned char *str;
   UuidToStringA (&uuid, &str);
 
-  char *ret = strdup ((char*)str);
+  char *ret = xstrdup ((char*)str);
   RpcStringFreeA (&str);
 
   return ret;
@@ -1755,7 +1958,7 @@ get_unique_id (LPDISPATCH mail, int create, const char *uuid)
     }
   else
     {
-      newuid = strdup (uuid);
+      newuid = xstrdup (uuid);
     }
   int ret = put_pa_string (mail, GPGOL_UID_DASL, newuid);
 
@@ -1886,6 +2089,7 @@ get_account_for_mail (const char *mbox)
 
       if (!smtpAddr)
         {
+          gpgol_release (account);
           TRACEPOINT;
           continue;
         }
@@ -1895,6 +2099,7 @@ get_account_for_mail (const char *mbox)
           xfree (smtpAddr);
           return account;
         }
+      gpgol_release (account);
       xfree (smtpAddr);
     }
   gpgol_release (accounts);
@@ -2007,6 +2212,21 @@ get_sender_SenderEMailAddress (LPDISPATCH mailitem)
   return nullptr;
 }
 
+char *
+get_sender_SentRepresentingAddress (LPDISPATCH mailitem)
+{
+  char *buf = get_pa_string (mailitem,
+                             PR_SENT_REPRESENTING_EMAIL_ADDRESS_W_DASL);
+  if (buf && strlen (buf))
+    {
+      log_debug ("%s:%s Found sent representing address \"%s\"",
+                 SRCNAME, __func__, buf);
+      return buf;
+    }
+  xfree (buf);
+  return nullptr;
+}
+
 char *
 get_inline_body ()
 {
@@ -2061,3 +2281,144 @@ get_ex_major_version_for_addr (const char *mbox)
 
   return (int) version;
 }
+
+int
+get_ol_ui_language ()
+{
+  LPDISPATCH app = GpgolAddin::get_instance()->get_application();
+  if (!app)
+    {
+      TRACEPOINT;
+      return 0;
+    }
+
+  LPDISPATCH langSettings = get_oom_object (app, "LanguageSettings");
+  if (!langSettings)
+    {
+      TRACEPOINT;
+      return 0;
+    }
+
+  VARIANT var;
+  VariantInit (&var);
+
+  VARIANT aVariant[1];
+  DISPPARAMS dispparams;
+
+  dispparams.rgvarg = aVariant;
+  dispparams.rgvarg[0].vt = VT_INT;
+  dispparams.rgvarg[0].intVal = 2;
+  dispparams.cArgs = 1;
+  dispparams.cNamedArgs = 0;
+
+  int ret = invoke_oom_method_with_parms_type (langSettings, "LanguageID", &var,
+                                               &dispparams,
+                                               DISPATCH_PROPERTYGET);
+  gpgol_release (langSettings);
+  if (ret)
+    {
+      TRACEPOINT;
+      return 0;
+    }
+  if (var.vt != VT_INT && var.vt != VT_I4)
+    {
+      TRACEPOINT;
+      return 0;
+    }
+
+  int result = var.intVal;
+
+  VariantClear (&var);
+  return result;
+}
+
+void
+log_addins ()
+{
+  LPDISPATCH app = GpgolAddin::get_instance ()->get_application ();
+
+  if (!app)
+    {
+      TRACEPOINT;
+      return;
+   }
+
+  LPDISPATCH addins = get_oom_object (app, "COMAddins");
+
+  if (!addins)
+    {
+      TRACEPOINT;
+      return;
+    }
+
+  std::string activeAddins;
+  int count = get_oom_int (addins, "Count");
+  for (int i = 1; i <= count; i++)
+    {
+      std::string item = std::string ("Item(") + std::to_string (i) + ")";
+
+      LPDISPATCH addin = get_oom_object (addins, item.c_str ());
+
+      if (!addin)
+        {
+          TRACEPOINT;
+          continue;
+        }
+      bool connected = get_oom_bool (addin, "Connect");
+      if (!connected)
+        {
+          gpgol_release (addin);
+          continue;
+        }
+
+      char *progId = get_oom_string (addin, "ProgId");
+      gpgol_release (addin);
+
+      if (!progId)
+        {
+          TRACEPOINT;
+          continue;
+        }
+      activeAddins += std::string (progId) + "\n";
+      xfree (progId);
+    }
+  gpgol_release (addins);
+
+  log_debug ("%s:%s:Active Addins:\n%s", SRCNAME, __func__,
+             activeAddins.c_str ());
+  return;
+}
+
+bool
+is_preview_pane_visible (LPDISPATCH explorer)
+{
+  if (!explorer)
+    {
+      TRACEPOINT;
+      return false;
+    }
+  VARIANT var;
+  VariantInit (&var);
+  VARIANT argvars[1];
+  DISPPARAMS args;
+  VariantInit (&argvars[0]);
+  argvars[0].vt = VT_INT;
+  argvars[0].intVal = 3;
+  args.cArgs = 1;
+  args.cNamedArgs = 0;
+  args.rgvarg = argvars;
+
+  if (invoke_oom_method_with_parms (explorer, "IsPaneVisible", &var, &args))
+    {
+      log_error ("%s:%s: Failed to check visibilty.",
+                 SRCNAME, __func__);
+      return false;
+    }
+
+  if (var.vt != VT_BOOL)
+    {
+      TRACEPOINT;
+      return false;
+    }
+  return !!var.boolVal;
+}
