@@ -25,6 +25,8 @@
 #include "attachment.h"
 #include "mimedataprovider.h"
 
+#include "keycache.h"
+
 #include <gpgme++/context.h>
 #include <gpgme++/decryptionresult.h>
 #include <gpgme++/key.h>
@@ -80,6 +82,7 @@ ParseController::ParseController(LPSTREAM instream, msgtype_t type):
     m_type (type),
     m_block_html (false)
 {
+  memdbg_ctor ("ParseController");
   log_mime_parser ("%s:%s: Creating parser for stream: %p of type %i"
                    " expect no headers: %i expect no mime: %i",
                    SRCNAME, __func__, instream, type,
@@ -94,6 +97,7 @@ ParseController::ParseController(FILE *instream, msgtype_t type):
     m_type (type),
     m_block_html (false)
 {
+  memdbg_ctor ("ParseController");
   log_mime_parser ("%s:%s: Creating parser for stream: %p of type %i",
                    SRCNAME, __func__, instream, type);
 }
@@ -101,6 +105,7 @@ ParseController::ParseController(FILE *instream, msgtype_t type):
 ParseController::~ParseController()
 {
   log_debug ("%s:%s", SRCNAME, __func__);
+  memdbg_dtor ("ParseController");
   delete m_inputprovider;
   delete m_outputprovider;
 }
@@ -223,6 +228,7 @@ format_error(GpgME::DecryptionResult result, Protocol protocol)
                  SRCNAME, __func__);
       return "Failed to Format error.";
     }
+  memdbg_alloc (buf);
   msg = buf;
   return msg;
 }
@@ -236,22 +242,33 @@ ParseController::setSender(const std::string &sender)
 static bool
 is_valid_chksum(const GpgME::Signature &sig)
 {
-  switch (sig.summary())
-    {
-      case GpgME::Signature::Valid:
-      case GpgME::Signature::Green:
-      case GpgME::Signature::KeyRevoked:
-      case GpgME::Signature::KeyExpired:
-      case GpgME::Signature::SigExpired:
-      case GpgME::Signature::CrlMissing:
-      case GpgME::Signature::CrlTooOld:
-      case GpgME::Signature::TofuConflict:
-        return true;
-      default:
-        return false;
-    }
+  const auto sum = sig.summary();
+  static unsigned int valid_mask = (unsigned int) (
+      GpgME::Signature::Valid |
+      GpgME::Signature::Green |
+      GpgME::Signature::KeyRevoked |
+      GpgME::Signature::KeyExpired |
+      GpgME::Signature::SigExpired |
+      GpgME::Signature::CrlMissing |
+      GpgME::Signature::CrlTooOld |
+      GpgME::Signature::TofuConflict );
+
+  return sum & valid_mask;
 }
 
+
+/* Note on stability:
+
+   Experiments have shown that we can have a crash if parse
+   returns at time that is not good for the state of Outlook.
+
+   This happend in my test instance after a delay of > 1s < 3s
+   with a < 1% chance :-/
+
+   So if you have really really bad luck this might still crash
+   although it usually should be either much quicker or much slower
+   (slower e.g. when pinentry is requrired).
+*/
 void
 ParseController::parse()
 {
@@ -282,7 +299,7 @@ ParseController::parse()
     {
       protocol = Protocol::OpenPGP;
     }
-  auto ctx = Context::createForProtocol (protocol);
+  auto ctx = std::unique_ptr<Context> (Context::createForProtocol (protocol));
   if (!ctx)
     {
       log_error ("%s:%s:Failed to create context. Installation broken.",
@@ -301,10 +318,18 @@ ParseController::parse()
           /* Should never happen */
           m_error = std::string("Bad installation");
         }
+      memdbg_alloc (buf);
       m_error = buf;
       xfree (buf);
       return;
     }
+
+  /* Maybe a different option for this ? */
+  if (opt.autoresolve)
+    {
+      ctx->setFlag("auto-key-retrieve", "1");
+    }
+
   ctx->setArmor(true);
 
   if (!m_sender.empty())
@@ -387,7 +412,7 @@ ParseController::parse()
                   break;
                 }
             }
-
+#ifdef HAVE_W32_SYSTEM
           if (allBad)
             {
               log_debug ("%s:%s:%p inline verify error trying native to utf8.",
@@ -410,8 +435,7 @@ ParseController::parse()
               if (utf8)
                 {
                   // Try again after conversion.
-                  delete ctx;
-                  ctx = Context::createForProtocol (protocol);
+                  ctx = std::unique_ptr<Context> (Context::createForProtocol (protocol));
                   ctx->setArmor (true);
                   if (!m_sender.empty())
                     {
@@ -435,29 +459,33 @@ ParseController::parse()
                   m_verify_result = ctx->verifyOpaqueSignature(input, output);
                 }
             }
+#endif
         }
     }
-  delete ctx;
   log_debug ("%s:%s:%p: decrypt err: %i verify err: %i",
              SRCNAME, __func__, this, m_decrypt_result.error().code(),
              m_verify_result.error().code());
 
-  TRACEPOINT;
-
   bool has_valid_encrypted_checksum = false;
   /* Ensure that the Keys for the signatures are available
      and if it has a valid encrypted checksum. */
+  bool ultimate_keys_queried = false;
   for (const auto sig: m_verify_result.signatures())
     {
       has_valid_encrypted_checksum = is_valid_chksum (sig);
 
-      sig.key(true, true);
-      if (sig.validity() == Signature::Validity::Full ||
-          sig.validity() == Signature::Validity::Ultimate)
+      KeyCache::instance ()->update (sig.fingerprint (), protocol);
+
+      if (!ultimate_keys_queried &&
+          (sig.validity() == Signature::Validity::Full ||
+          sig.validity() == Signature::Validity::Ultimate))
         {
           /* Ensure that we have the keys with ultimate
              trust cached for the ui. */
+
+          // TODO this is something for the keycache
           get_ultimate_keys ();
+          ultimate_keys_queried = true;
         }
     }
 
@@ -471,11 +499,20 @@ ParseController::parse()
 
   if (opt.enable_debug)
     {
-       std::stringstream ss;
-       ss << m_decrypt_result << '\n' << m_verify_result;
+      std::stringstream ss;
+      ss << m_decrypt_result << '\n' << m_verify_result;
       for (const auto sig: m_verify_result.signatures())
         {
-          ss << '\n' << sig.key();
+          const auto key = sig.key();
+          if (key.isNull())
+            {
+              ss << '\n' << "Cached key:\n" << KeyCache::instance()->getByFpr(
+                  sig.fingerprint(), false);
+            }
+          else
+            {
+              ss << '\n' << key;
+            }
         }
        log_debug ("Decrypt / Verify result: %s", ss.str().c_str());
     }
@@ -485,11 +522,11 @@ ParseController::parse()
 }
 
 const std::string
-ParseController::get_html_body() const
+ParseController::get_html_body () const
 {
   if (m_outputprovider)
     {
-      return m_outputprovider->get_html_body();
+      return m_outputprovider->get_html_body ();
     }
   else
     {
@@ -498,11 +535,11 @@ ParseController::get_html_body() const
 }
 
 const std::string
-ParseController::get_body() const
+ParseController::get_body () const
 {
   if (m_outputprovider)
     {
-      return m_outputprovider->get_body();
+      return m_outputprovider->get_body ();
     }
   else
     {
@@ -564,7 +601,7 @@ ParseController::get_ultimate_keys()
     }
   log_debug ("%s:%s: Starting keylisting.",
              SRCNAME, __func__);
-  auto ctx = Context::createForProtocol (OpenPGP);
+  auto ctx = std::unique_ptr<Context> (Context::createForProtocol (OpenPGP));
   if (!ctx)
     {
       /* Maybe PGP broken and not S/MIME */
@@ -580,7 +617,6 @@ ParseController::get_ultimate_keys()
     {
       log_error ("%s:%s: Failed to start keylisting err: %i: %s",
                  SRCNAME, __func__, err.code (), err.asString());
-      delete ctx;
       gpgrt_lock_unlock (&keylist_lock);
       return s_ultimate_keys;
     }
@@ -614,7 +650,6 @@ ParseController::get_ultimate_keys()
         }
     }
   TRACEPOINT;
-  delete ctx;
   log_debug ("%s:%s: keylisting done.",
              SRCNAME, __func__);
 
