@@ -122,7 +122,7 @@ rfc2047_token_new_encoded_word (const char *word, size_t len)
 
   /* copy the charset into a buffer */
   n = (size_t) (inptr - tmpchar);
-  buf = malloc (n + 1);
+  buf = xmalloc (n + 1);
   memcpy (buf, tmpchar, n);
   buf[n] = '\0';
   charset = buf;
@@ -556,7 +556,7 @@ rfc2047_decode_tokens (rfc2047_token *tokens, size_t buflen)
           /* make sure our temporary output buffer is large enough... */
           if (len > tmplen)
             {
-              xrealloc (outbuf, len + 1);
+              outbuf = xrealloc (outbuf, len + 1);
               tmplen = len + 1;
             }
 
@@ -652,7 +652,7 @@ rfc2047_parse (const char *input)
 {
   char *decoded;
   if (!input)
-    return strdup ("");
+    return xstrdup ("");
 
   log_debug ("%s:%s: Input: \"%s\"",
              SRCNAME, __func__, input);
@@ -665,7 +665,7 @@ rfc2047_parse (const char *input)
   if (!decoded || !strlen (decoded))
     {
       xfree (decoded);
-      return strdup (input);
+      return xstrdup (input);
     }
   return decoded;
 }
