@@ -42,12 +42,22 @@
 #include "mimemaker.h"
 #include "filetype.h"
 #include "mail.h"
+#include "dispcache.h"
 
 #include <gpgme++/context.h>
 #include <gpgme++/data.h>
 
+#undef _
+#define _(a) utf8_gettext (a)
+
 using namespace GpgME;
 
+/* This is so super stupid. I bet even Microsft developers laugh
+   about the definition of VARIANT_BOOL. And then for COM we
+   have to pass pointers to this stuff. */
+static VARIANT_BOOL var_true = VARIANT_TRUE;
+static VARIANT_BOOL var_false = VARIANT_FALSE;
+
 /* Gets the context of a ribbon control. And prints some
    useful debug output */
 HRESULT getContext (LPDISPATCH ctrl, LPDISPATCH *context)
@@ -64,15 +74,15 @@ HRESULT getContext (LPDISPATCH ctrl, LPDISPATCH *context)
   return context ? S_OK : E_FAIL;
 }
 
-/* getIcon
+/* getIconDisp
    Loads a PNG image from the resurce converts it into a Bitmap
    and Wraps it in an PictureDispatcher that is returned as result.
 
    Based on documentation from:
    http://www.codeproject.com/Articles/3537/Loading-JPG-PNG-resources-using-GDI
 */
-HRESULT
-getIcon (int id, VARIANT* result)
+static LPDISPATCH
+getIconDisp (int id)
 {
   PICTDESC pdesc;
   LPDISPATCH pPict;
@@ -89,12 +99,6 @@ getIcon (int id, VARIANT* result)
   pdesc.cbSizeofstruct = sizeof pdesc;
   pdesc.picType = PICTYPE_BITMAP;
 
-  if (!result)
-    {
-      log_error ("getIcon called without result variant.");
-      return E_POINTER;
-    }
-
   /* Initialize GDI */
   gdiplusStartupInput.DebugEventCallback = NULL;
   gdiplusStartupInput.SuppressBackgroundThread = FALSE;
@@ -108,12 +112,12 @@ getIcon (int id, VARIANT* result)
     {
       log_error ("%s:%s: failed to find image: %i",
                  SRCNAME, __func__, id);
-      return E_FAIL;
+      return nullptr;
     }
 
   imageSize = SizeofResource (glob_hinst, hResource);
   if (!imageSize)
-    return E_FAIL;
+    return nullptr;
 
   pResourceData = LockResource (LoadResource(glob_hinst, hResource));
 
@@ -121,7 +125,7 @@ getIcon (int id, VARIANT* result)
     {
       log_error ("%s:%s: failed to load image: %i",
                  SRCNAME, __func__, id);
-      return E_FAIL;
+      return nullptr;
     }
 
   hBuffer = GlobalAlloc (GMEM_MOVEABLE, imageSize);
@@ -134,8 +138,9 @@ getIcon (int id, VARIANT* result)
           IStream* pStream = NULL;
           CopyMemory (pBuffer, pResourceData, imageSize);
 
-          if (CreateStreamOnHGlobal (hBuffer, FALSE, &pStream) == S_OK)
+          if (CreateStreamOnHGlobal (hBuffer, TRUE, &pStream) == S_OK)
             {
+              memdbg_addRef (pStream);
               pbitmap = Gdiplus::Bitmap::FromStream (pStream);
               gpgol_release (pStream);
               if (!pbitmap || pbitmap->GetHBITMAP (0, &pdesc.bmp.hbitmap))
@@ -158,11 +163,36 @@ getIcon (int id, VARIANT* result)
     {
       log_error ("%s:%s: OleCreatePictureIndirect failed: hr=%#lx\n",
                  SRCNAME, __func__, hr);
-      return -1;
+      return nullptr;
     }
 
-  result->pdispVal = pPict;
-  result->vt = VT_DISPATCH;
+  return pPict;
+}
+
+HRESULT
+getIcon (int id, VARIANT* result)
+{
+  if (!result)
+    {
+      TRACEPOINT;
+      return S_OK;
+    }
+
+  auto cache = DispCache::instance ();
+  result->pdispVal = cache->getDisp (id);
+
+  if (!result->pdispVal)
+    {
+      result->pdispVal = getIconDisp (id);
+      cache->addDisp (id, result->pdispVal);
+      memdbg_addRef (result->pdispVal);
+    }
+
+  if (result->pdispVal)
+    {
+      result->vt = VT_DISPATCH;
+      result->pdispVal->AddRef();
+    }
 
   return S_OK;
 }
@@ -170,40 +200,71 @@ getIcon (int id, VARIANT* result)
 HRESULT
 mark_mime_action (LPDISPATCH ctrl, int flags, bool is_explorer)
 {
-  HRESULT hr;
-  HRESULT rc = E_FAIL;
-  LPDISPATCH context = NULL,
-             mailitem = NULL;
   LPMESSAGE message = NULL;
   int oldflags,
       newflags;
 
   log_debug ("%s:%s: enter", SRCNAME, __func__);
-  hr = getContext (ctrl, &context);
-  if (FAILED(hr))
-      return hr;
+  LPDISPATCH context = NULL;
+  if (FAILED(getContext (ctrl, &context)))
+    {
+      TRACEPOINT;
+      return E_FAIL;
+    }
 
-  mailitem = get_oom_object (context, is_explorer ? "ActiveInlineResponse" :
-                                                    "CurrentItem");
+  LPDISPATCH mailitem = get_oom_object (context,
+                                        is_explorer ? "ActiveInlineResponse" :
+                                                      "CurrentItem");
+  gpgol_release (context);
 
   if (!mailitem)
     {
       log_error ("%s:%s: Failed to get mailitem.",
                  SRCNAME, __func__);
-      goto done;
+      return E_FAIL;
+    }
+
+  /* Get the uid of this item. */
+  char *uid = get_unique_id (mailitem, 0, nullptr);
+  if (!uid)
+    {
+      LPMESSAGE msg = get_oom_base_message (mailitem);
+      uid = mapi_get_uid (msg);
+      gpgol_release (msg);
+      if (!uid)
+        {
+          log_debug ("%s:%s: Failed to get uid for %p",
+                   SRCNAME, __func__, mailitem);
+        }
+    }
+  Mail *mail = nullptr;
+  if (uid)
+    {
+      mail = Mail::getMailForUUID (uid);
+      xfree (uid);
+    }
+
+  if (mail)
+    {
+      mail->setCryptoSelectedManually (true);
+    }
+  else
+    {
+      log_debug ("%s:%s: Failed to get mail object.",
+                 SRCNAME, __func__);
     }
 
   message = get_oom_base_message (mailitem);
+  gpgol_release (mailitem);
 
   if (!message)
     {
       log_error ("%s:%s: Failed to get message.",
                  SRCNAME, __func__);
-      goto done;
+      return S_OK;
     }
 
   oldflags = get_gpgol_draft_info_flags (message);
-
   if (flags == 3 && oldflags != 3)
     {
       // If only one sub button is active activate
@@ -220,8 +281,7 @@ mark_mime_action (LPDISPATCH ctrl, int flags, bool is_explorer)
       log_error ("%s:%s: Failed to set draft flags.",
                  SRCNAME, __func__);
     }
-
-  rc = S_OK;
+  gpgol_release (message);
 
   /*  We need to invalidate the UI to update the toggle
       states of the subbuttons and the top button. Yeah,
@@ -230,15 +290,10 @@ mark_mime_action (LPDISPATCH ctrl, int flags, bool is_explorer)
 
   if (newflags & 1)
     {
-      Mail::locate_all_crypto_recipients ();
+      Mail::locateAllCryptoRecipients_o ();
     }
 
-done:
-  gpgol_release (context);
-  gpgol_release (mailitem);
-  gpgol_release (message);
-
-  return rc;
+  return S_OK;
 }
 
 /* Get the state of encrypt / sign toggle buttons.
@@ -256,8 +311,7 @@ HRESULT get_crypt_pressed (LPDISPATCH ctrl, int flags, VARIANT *result,
   LPMESSAGE message = NULL;
 
   result->vt = VT_BOOL | VT_BYREF;
-  result->pboolVal = (VARIANT_BOOL*) xmalloc (sizeof (VARIANT_BOOL));
-  *(result->pboolVal) = VARIANT_FALSE;
+  result->pboolVal = &var_false;
 
   /* First the usual defensive check about our parameters */
   if (!ctrl || !result)
@@ -296,7 +350,7 @@ HRESULT get_crypt_pressed (LPDISPATCH ctrl, int flags, VARIANT *result,
 
   value = (get_gpgol_draft_info_flags (message) & flags) == flags;
 
-  *(result->pboolVal) = value ? VARIANT_TRUE : VARIANT_FALSE;
+  result->pboolVal = value ? &var_true: &var_false;
 
 done:
   gpgol_release (context);
@@ -326,8 +380,15 @@ get_mail_from_control (LPDISPATCH ctrl, bool *none_selected)
                  hr);
       return NULL;
     }
+  char *name = get_object_name (context);
 
-  const auto ctx_name = std::string (get_object_name (context));
+  std::string ctx_name;
+
+  if (name)
+    {
+      ctx_name = name;
+      xfree (name);
+    }
 
   if (ctx_name.empty())
     {
@@ -343,19 +404,38 @@ get_mail_from_control (LPDISPATCH ctrl, bool *none_selected)
     }
   else if (!strcmp (ctx_name.c_str(), "_Explorer"))
     {
+      /* Avoid showing wrong crypto state if we don't have a reading
+         pane. In that case the parser will finish for a mail which is gone
+         and the crypto state will not get updated. */
+
+      if (!is_preview_pane_visible (context))
+        {
+          *none_selected = true;
+          gpgol_release (mailitem);
+          mailitem = nullptr;
+          log_debug ("%s:%s: Preview pane invisible", SRCNAME, __func__);
+        }
+
+#if 0
       if (g_ol_version_major >= 16)
         {
-          // Avoid showing wrong crypto state if we don't have a reading
-          // pane. In that case the parser will finish for a mail which is gone
-          // and the crypto state will not get updated.
-          //
-          //
-          // Somehow latest Outlook 2016 crashes when accessing the current view
-          // of the Explorer. This is even reproducible with
-          // GpgOL disabled and only with Outlook Spy active. If you select
-          // the explorer of an Outlook.com resource and then access
-          // the CurrentView and close the CurrentView again in Outlook Spy
-          // outlook crashes.
+          /* Some Versions of Outlook 2016 crashed when accessing the current view
+             of the Explorer. This was even reproducible with
+             GpgOL disabled and only with Outlook Spy active. If you selected
+             the explorer of an Outlook.com resource and then access
+             the CurrentView and close the CurrentView again in Outlook Spy
+             outlook crashes. See: T3484
+
+             The crash no longer occured at least since build 10228. As
+             I'm not sure which Version fixed the crash we don't do a version
+             check and just use the same codepath as for Outlook 2010 and
+             2013 again.
+
+             Accessing PreviewPane.WordEditor is not a good solution here as
+             it requires "Microsoft VBA for Office" (See T4056 ).
+             A possible solution for that might be to check if
+             "Mail.GetInspector().WordEditor()" returns NULL. In that case we
+             know that we also won't get a WordEditor in the preview pane. */
           LPDISPATCH prevEdit = get_oom_object (context, "PreviewPane.WordEditor");
           gpgol_release (prevEdit);
           if (!prevEdit)
@@ -389,6 +469,8 @@ get_mail_from_control (LPDISPATCH ctrl, bool *none_selected)
                 }
             }
         }
+#endif
+
       if (!*none_selected)
         {
           /* Accessing the selection item can trigger a load event
@@ -460,7 +542,7 @@ get_mail_from_control (LPDISPATCH ctrl, bool *none_selected)
         }
     }
 
-  auto ret = Mail::get_mail_for_uuid (uid);
+  auto ret = Mail::getMailForUUID (uid);
   xfree (uid);
   if (!ret)
     {
@@ -498,9 +580,9 @@ HRESULT get_is_details_enabled (LPDISPATCH ctrl, VARIANT *result)
     }
 
   result->vt = VT_BOOL | VT_BYREF;
-  result->pboolVal = (VARIANT_BOOL*) xmalloc (sizeof (VARIANT_BOOL));
-  *(result->pboolVal) = none_selected ? VARIANT_FALSE : VARIANT_TRUE;
+  result->pboolVal = none_selected ? &var_false : &var_true;
 
+  TRACEPOINT;
   return S_OK;
 }
 
@@ -519,9 +601,10 @@ HRESULT get_sig_label (LPDISPATCH ctrl, VARIANT *result)
       xfree (w_result);
       return S_OK;
     }
-  w_result = utf8_to_wchar (mail->get_crypto_summary ().c_str ());
+  w_result = utf8_to_wchar (mail->getCryptoSummary ().c_str ());
   result->bstrVal = SysAllocString (w_result);
   xfree (w_result);
+  TRACEPOINT;
   return S_OK;
 }
 
@@ -533,7 +616,7 @@ HRESULT get_sig_ttip (LPDISPATCH ctrl, VARIANT *result)
   wchar_t *w_result;
   if (mail)
     {
-      w_result = utf8_to_wchar (mail->get_crypto_one_line().c_str());
+      w_result = utf8_to_wchar (mail->getCryptoOneLine ().c_str());
     }
   else if (!none_selected)
     {
@@ -545,6 +628,7 @@ HRESULT get_sig_ttip (LPDISPATCH ctrl, VARIANT *result)
     }
   result->bstrVal = SysAllocString (w_result);
   xfree (w_result);
+  TRACEPOINT;
   return S_OK;
 }
 
@@ -556,9 +640,10 @@ HRESULT get_sig_stip (LPDISPATCH ctrl, VARIANT *result)
   if (none_selected)
     {
       result->bstrVal = SysAllocString (L"");
+      TRACEPOINT;
       return S_OK;
     }
-  if (!mail || !mail->is_crypto_mail ())
+  if (!mail || !mail->isCryptoMail ())
     {
       wchar_t *w_result;
       w_result = utf8_to_wchar (utf8_gettext ("You cannot be sure who sent, "
@@ -566,12 +651,14 @@ HRESULT get_sig_stip (LPDISPATCH ctrl, VARIANT *result)
                                   "Click here to learn more."));
       result->bstrVal = SysAllocString (w_result);
       xfree (w_result);
+      TRACEPOINT;
       return S_OK;
     }
-  const auto message = mail->get_crypto_details ();
+  const auto message = mail->getCryptoDetails_o ();
   wchar_t *w_message = utf8_to_wchar (message.c_str());
   result->bstrVal = SysAllocString (w_message);
   xfree (w_message);
+  TRACEPOINT;
   return S_OK;
 }
 
@@ -579,14 +666,14 @@ HRESULT launch_cert_details (LPDISPATCH ctrl)
 {
   MY_MAIL_GETTER
 
-  if (!mail || (!mail->is_signed () && !mail->is_encrypted ()))
+  if (!mail || (!mail->isSigned () && !mail->isEncrypted ()))
     {
       ShellExecuteA(NULL, NULL, "https://emailselfdefense.fsf.org/infographic",
                     0, 0, SW_SHOWNORMAL);
       return S_OK;
     }
 
-  if (!mail->is_signed () && mail->is_encrypted ())
+  if (!mail->isSigned () && mail->isEncrypted ())
     {
       /* Encrypt only, no information but show something. because
          we want the button to be active.
@@ -600,18 +687,18 @@ HRESULT launch_cert_details (LPDISPATCH ctrl)
       char * buf;
       gpgrt_asprintf (&buf, _("The message was not cryptographically signed.\n"
                       "There is no additional information available if it "
-                      "was actually sent by '%s' or if someone faked the sender address."), mail->get_sender ().c_str());
+                      "was actually sent by '%s' or if someone faked the sender address."), mail->getSender_o ().c_str());
       MessageBox (NULL, buf, _("GpgOL"),
                   MB_ICONINFORMATION|MB_OK);
       xfree (buf);
       return S_OK;
     }
 
-  if (!mail->get_sig_fpr())
+  if (!mail->getSigFpr ())
     {
       std::string buf = _("There was an error verifying the signature.\n"
                            "Full details:\n");
-      buf += mail->get_verification_result_dump();
+      buf += mail->getVerificationResultDump ();
       gpgol_message_box (get_active_hwnd(), buf.c_str(), _("GpgOL"), MB_OK);
     }
 
@@ -637,7 +724,7 @@ HRESULT launch_cert_details (LPDISPATCH ctrl)
             std::string parentWid = std::to_string ((int) (intptr_t) get_active_hwnd ());
             const char *argv[] = {path.c_str(),
                                   "--query",
-                                  mail->get_sig_fpr(),
+                                  mail->getSigFpr (),
                                   "--parent-windowid",
                                   parentWid.c_str(),
                                   NULL };
@@ -676,8 +763,10 @@ HRESULT get_crypto_icon (LPDISPATCH ctrl, VARIANT *result)
 
   if (mail)
     {
-      return getIcon (mail->get_crypto_icon_id (), result);
+      TRACEPOINT;
+      return getIcon (mail->getCryptoIconID (), result);
     }
+  TRACEPOINT;
   return getIcon (IDI_LEVEL_0, result);
 }
 
@@ -686,10 +775,10 @@ HRESULT get_is_crypto_mail (LPDISPATCH ctrl, VARIANT *result)
   MY_MAIL_GETTER
 
   result->vt = VT_BOOL | VT_BYREF;
-  result->pboolVal = (VARIANT_BOOL*) xmalloc (sizeof (VARIANT_BOOL));
-  *(result->pboolVal) = (mail && (mail->is_signed () || mail->is_encrypted ())) ?
-                          VARIANT_TRUE : VARIANT_FALSE;
+  result->pboolVal = mail && (mail->isSigned () || mail->isEncrypted ()) ?
+                          &var_true : &var_false;
 
+  TRACEPOINT;
   return S_OK;
 }
 
