@@ -25,6 +25,13 @@
 # define LC_MESSAGES 1729
 #endif
 
+#ifdef __cplusplus
+extern "C" {
+#if 0
+}
+#endif
+#endif
+
 /* Specify that the DOMAINNAME message catalog will be found
    in DIRNAME rather than in the system locale data base.  */
 char *bindtextdomain (const char *domainname, const char *dirname);
@@ -57,11 +64,33 @@ static inline const char *gettext_localename (void) { return ""; }
 
 #endif	/* !ENABLE_NLS */
 
-/* Conversion function. */
-char *wchar_to_utf8 (const wchar_t *string);
-wchar_t *utf8_to_wchar (const char *string);
 
+/* Conversion function. */
+char *_wchar_to_utf8 (const wchar_t *string);
+wchar_t *_utf8_to_wchar (const char *string);
 char *utf8_to_native (const char *string);
 char *native_to_utf8 (const char *string);
 
+#define utf8_to_wchar(VAR1) ({wchar_t *retval; \
+  retval = _utf8_to_wchar (VAR1); \
+  if ((opt.enable_debug & DBG_OOM_EXTRA) && 0) \
+  { \
+    log_debug ("%s:%s:%i wchar_t alloc %p:%S", \
+               SRCNAME, __func__, __LINE__, retval, retval); \
+  } \
+retval;})
 
+#define wchar_to_utf8(VAR1) ({char *retval; \
+  retval = _wchar_to_utf8 (VAR1); \
+  if ((opt.enable_debug & DBG_OOM_EXTRA) && 0) \
+  { \
+    log_debug ("%s:%s:%i char utf8 alloc %p:%s", \
+               SRCNAME, __func__, __LINE__, retval, retval); \
+  } \
+retval;})
+
+#ifdef __cplusplus
+}
+#include <string>
+std::string wchar_to_utf8_string (const wchar_t *string);
+#endif
