@@ -33,10 +33,12 @@
 
 /* Singleton window */
 static HWND g_responder_window = NULL;
+static int invalidation_blocked = 0;
 
 LONG_PTR WINAPI
 gpgol_window_proc (HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 {
+//  log_debug ("WMG: %x", (unsigned int) message);
   if (message == WM_USER + 42)
     {
       wm_ctx_t *ctx = (wm_ctx_t *) lParam;
@@ -47,7 +49,7 @@ gpgol_window_proc (HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
           case (PARSING_DONE):
             {
               auto mail = (Mail*) ctx->data;
-              if (!Mail::is_valid_ptr (mail))
+              if (!Mail::isValidPtr (mail))
                 {
                   log_debug ("%s:%s: Parsing done for mail which is gone.",
                              SRCNAME, __func__);
@@ -59,26 +61,26 @@ gpgol_window_proc (HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
           case (RECIPIENT_ADDED):
             {
               auto mail = (Mail*) ctx->data;
-              if (!Mail::is_valid_ptr (mail))
+              if (!Mail::isValidPtr (mail))
                 {
                   log_debug ("%s:%s: Recipient add for mail which is gone.",
                              SRCNAME, __func__);
                   break;
                 }
-              mail->locate_keys();
+              mail->locateKeys_o ();
               break;
             }
           case (REVERT_MAIL):
             {
               auto mail = (Mail*) ctx->data;
-              if (!Mail::is_valid_ptr (mail))
+              if (!Mail::isValidPtr (mail))
                 {
                   log_debug ("%s:%s: Revert mail for mail which is gone.",
                              SRCNAME, __func__);
                   break;
                 }
 
-              mail->set_needs_save (true);
+              mail->setNeedsSave (true);
               /* Some magic here. Accessing any existing inline body cements
                  it. Otherwise updating the body through the revert also changes
                  the body of a inline mail. */
@@ -91,57 +93,70 @@ gpgol_window_proc (HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
               invoke_oom_method (mail->item (), "Save", NULL);
               log_debug ("%s:%s: Revert mail. Save done. Updating body..",
                          SRCNAME, __func__);
-              mail->update_body ();
+              mail->updateBody_o ();
               log_debug ("%s:%s: Revert mail done.",
                          SRCNAME, __func__);
               break;
             }
           case (INVALIDATE_UI):
             {
-              log_debug ("%s:%s: Invalidating UI",
-                         SRCNAME, __func__);
-              gpgoladdin_invalidate_ui();
-              log_debug ("%s:%s: Invalidation done",
-                         SRCNAME, __func__);
+              if (!invalidation_blocked)
+                {
+                  log_debug ("%s:%s: Invalidating UI",
+                             SRCNAME, __func__);
+                  gpgoladdin_invalidate_ui();
+                  log_debug ("%s:%s: Invalidation done",
+                             SRCNAME, __func__);
+                }
+              else
+                {
+                  log_debug ("%s:%s: Received invalidation msg while blocked."
+                             " Ignoring it",
+                             SRCNAME, __func__);
+                }
               break;
             }
           case (INVALIDATE_LAST_MAIL):
             {
-              log_debug ("%s:%s: Invalidating last mail",
+              log_debug ("%s:%s: clearing last mail",
                          SRCNAME, __func__);
-              Mail::invalidate_last_mail ();
+              Mail::clearLastMail ();
               break;
             }
           case (CLOSE):
             {
               auto mail = (Mail*) ctx->data;
-              if (!Mail::is_valid_ptr (mail))
+              if (!Mail::isValidPtr (mail))
                 {
                   log_debug ("%s:%s: Close for mail which is gone.",
                              SRCNAME, __func__);
                   break;
                 }
+              mail->refCurrentItem();
               Mail::close (mail);
+              log_debug ("%s:%s: Close finished.",
+                         SRCNAME, __func__);
+              mail->releaseCurrentItem();
               break;
             }
           case (CRYPTO_DONE):
             {
               auto mail = (Mail*) ctx->data;
-              if (!Mail::is_valid_ptr (mail))
+              if (!Mail::isValidPtr (mail))
                 {
                   log_debug ("%s:%s: Crypto done for mail which is gone.",
                              SRCNAME, __func__);
                   break;
                 }
               // modify the mail.
-              if (mail->crypt_state () == Mail::NeedsUpdateInOOM)
+              if (mail->cryptState () == Mail::NeedsUpdateInOOM)
                 {
                   // Save the Mail
                   log_debug ("%s:%s: Crypto done for %p updating oom.",
                              SRCNAME, __func__, mail);
-                  mail->update_crypt_oom();
+                  mail->updateCryptOOM_o ();
                 }
-              if (mail->crypt_state () == Mail::NeedsSecondAfterWrite)
+              if (mail->cryptState () == Mail::NeedsSecondAfterWrite)
                 {
                   invoke_oom_method (mail->item (), "Save", NULL);
                   log_debug ("%s:%s: Second save done for %p Invoking second send.",
@@ -151,17 +166,11 @@ gpgol_window_proc (HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
               invoke_oom_method (mail->item (), "Send", NULL);
               log_debug ("%s:%s:  Send for %p completed.",
                          SRCNAME, __func__, mail);
-              // Allow the WKS helper to queue a notification.
-              WKSHelper::instance()->allow_notify ();
-              log_debug ("%s:%s:  Crypto done handler completed.",
-                         SRCNAME, __func__);
+              mail->releaseCurrentItem();
               break;
             }
           case (BRING_TO_FRONT):
             {
-              // We want to avoid a race where the mail window
-              // was still the active window.
-              Sleep (250);
               HWND wnd = get_active_hwnd ();
               if (wnd)
                 {
@@ -185,21 +194,45 @@ gpgol_window_proc (HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
           case (CLEAR_REPLY_FORWARD):
             {
               auto mail = (Mail*) ctx->data;
-              if (!Mail::is_valid_ptr (mail))
+              if (!Mail::isValidPtr (mail))
                 {
                   log_debug ("%s:%s: Clear reply forward for mail which is gone.",
                              SRCNAME, __func__);
                   break;
                 }
-              mail->wipe (true);
-              mail->remove_all_attachments ();
+              mail->wipe_o (true);
+              mail->removeAllAttachments_o ();
+              break;
+            }
+          case (DO_AUTO_SECURE):
+            {
+              auto mail = (Mail*) ctx->data;
+              if (!Mail::isValidPtr (mail))
+                {
+                  log_debug ("%s:%s: DO_AUTO_SECURE for mail which is gone.",
+                             SRCNAME, __func__);
+                  break;
+                }
+              mail->setDoAutosecure_m (true);
+              break;
+            }
+          case (DONT_AUTO_SECURE):
+            {
+              auto mail = (Mail*) ctx->data;
+              if (!Mail::isValidPtr (mail))
+                {
+                  log_debug ("%s:%s: DO_AUTO_SECURE for mail which is gone.",
+                             SRCNAME, __func__);
+                  break;
+                }
+              mail->setDoAutosecure_m (false);
               break;
             }
           default:
             log_debug ("%s:%s: Unknown msg %x",
                        SRCNAME, __func__, ctx->wmsg_type);
         }
-        return DefWindowProc(hWnd, message, wParam, lParam);
+        return 0;
     }
   return DefWindowProc(hWnd, message, wParam, lParam);
 }
@@ -234,7 +267,7 @@ create_responder_window ()
   return g_responder_window;
 }
 
-int
+static int
 send_msg_to_ui_thread (wm_ctx_t *ctx)
 {
   size_t cls_name_len = strlen(RESPONDER_CLASS_NAME) + 1;
@@ -258,6 +291,10 @@ do_in_ui_thread (gpgol_wmsg_type type, void *data)
   wm_ctx_t ctx = {NULL, UNKNOWN, 0, 0};
   ctx.wmsg_type = type;
   ctx.data = data;
+
+  log_debug ("%s:%s: Sending message of type %i",
+             SRCNAME, __func__, type);
+
   if (send_msg_to_ui_thread (&ctx))
     {
       return -1;
@@ -284,7 +321,7 @@ do_async (LPVOID arg)
 void
 do_in_ui_thread_async (gpgol_wmsg_type type, void *data, int delay)
 {
-  wm_ctx_t *ctx = (wm_ctx_t *) calloc (1, sizeof (wm_ctx_t));
+  wm_ctx_t *ctx = (wm_ctx_t *) xcalloc (1, sizeof (wm_ctx_t));
   ctx->wmsg_type = type;
   ctx->data = data;
   ctx->delay = delay;
@@ -292,22 +329,6 @@ do_in_ui_thread_async (gpgol_wmsg_type type, void *data, int delay)
   CloseHandle (CreateThread (NULL, 0, do_async, (LPVOID) ctx, 0, NULL));
 }
 
-static std::vector <LPDISPATCH> explorers;
-
-void
-add_explorer (LPDISPATCH explorer)
-{
-  explorers.push_back (explorer);
-}
-
-void remove_explorer (LPDISPATCH explorer)
-{
-  explorers.erase(std::remove(explorers.begin(),
-                              explorers.end(),
-                              explorer),
-                  explorers.end());
-}
-
 LRESULT CALLBACK
 gpgol_hook(int code, WPARAM wParam, LPARAM lParam)
 {
@@ -317,36 +338,73 @@ gpgol_hook(int code, WPARAM wParam, LPARAM lParam)
    before it reaches outlook. */
   LPCWPSTRUCT cwp = (LPCWPSTRUCT) lParam;
 
+  /* What we do here is that we catch all WM_CLOSE messages that
+     get to Outlook. Then we check if the last open Explorer
+     is the target of the close. In set case we start our shutdown
+     routine before we pass the WM_CLOSE to outlook */
   switch (cwp->message)
     {
       case WM_CLOSE:
       {
         HWND lastChild = NULL;
-        for (const auto explorer: explorers)
+        log_debug ("%s:%s: Got WM_CLOSE",
+                   SRCNAME, __func__);
+        if (!GpgolAddin::get_instance() || !GpgolAddin::get_instance ()->get_application())
           {
-            /* Casting to LPOLEWINDOW and calling GetWindow
-               succeeded in Outlook 2016 but always returned
-               the number 1. So we need this hack. */
-            char *caption = get_oom_string (explorer, "Caption");
-            if (!caption)
-              {
-                log_debug ("%s:%s: No caption.",
-                           SRCNAME, __func__);
-                continue;
-              }
-            /* rctrl_renwnd32 is the window class of outlook. */
-            HWND hwnd = FindWindowExA(NULL, lastChild, "rctrl_renwnd32",
-                                      caption);
-            xfree (caption);
-            lastChild = hwnd;
-            if (hwnd == cwp->hwnd)
-              {
-                log_debug ("%s:%s: WM_CLOSE windowmessage for explorer. "
-                           "Closing all mails.",
-                           SRCNAME, __func__);
-                Mail::close_all_mails();
-                break;
-              }
+            TRACEPOINT;
+            break;
+          }
+        LPDISPATCH explorers = get_oom_object (GpgolAddin::get_instance ()->get_application(),
+                                               "Explorers");
+
+        if (!explorers)
+          {
+            log_error ("%s:%s: No explorers object",
+                       SRCNAME, __func__);
+            break;
+          }
+        int count = get_oom_int (explorers, "Count");
+
+        if (count != 1)
+          {
+            log_debug ("%s:%s: More then one explorer. Not shutting down.",
+                       SRCNAME, __func__);
+            gpgol_release (explorers);
+            break;
+          }
+
+        LPDISPATCH explorer = get_oom_object (explorers, "Item(1)");
+        gpgol_release (explorers);
+
+        if (!explorer)
+          {
+            TRACEPOINT;
+            break;
+          }
+
+        /* Casting to LPOLEWINDOW and calling GetWindow
+           succeeded in Outlook 2016 but always returned
+           the number 1. So we need this hack. */
+        char *caption = get_oom_string (explorer, "Caption");
+        gpgol_release (explorer);
+        if (!caption)
+          {
+            log_debug ("%s:%s: No caption.",
+                       SRCNAME, __func__);
+            break;
+          }
+        /* rctrl_renwnd32 is the window class of outlook. */
+        HWND hwnd = FindWindowExA(NULL, lastChild, "rctrl_renwnd32",
+                                  caption);
+        xfree (caption);
+        lastChild = hwnd;
+        if (hwnd == cwp->hwnd)
+          {
+            log_debug ("%s:%s: WM_CLOSE windowmessage for explorer. "
+                       "Shutting down.",
+                       SRCNAME, __func__);
+            GpgolAddin::get_instance ()->shutdown();
+            break;
           }
         break;
       }
@@ -358,10 +416,11 @@ gpgol_hook(int code, WPARAM wParam, LPARAM lParam)
         {
           log_debug ("%s:%s: SC_CLOSE syscommand. Closing all mails.",
                      SRCNAME, __func__);
-          Mail::close_all_mails();
+          GpgolAddin::get_instance ()->shutdown();
         } */
        break;
      default:
+//       log_debug ("WM: %x", (unsigned int) cwp->message);
        break;
     }
   return CallNextHookEx (NULL, code, wParam, lParam);
@@ -379,12 +438,12 @@ create_message_hook()
                            GetCurrentThreadId());
 }
 
-gpgrt_lock_t invalidate_lock = GPGRT_LOCK_INITIALIZER;
+GPGRT_LOCK_DEFINE (invalidate_lock);
 
 static bool invalidation_in_progress;
 
 DWORD WINAPI
-delayed_invalidate_ui (LPVOID)
+delayed_invalidate_ui (LPVOID minsleep)
 {
   if (invalidation_in_progress)
     {
@@ -392,13 +451,27 @@ delayed_invalidate_ui (LPVOID)
                  SRCNAME, __func__);
       return 0;
     }
-  gpgrt_lock_lock(&invalidate_lock);
+  TRACEPOINT;
   invalidation_in_progress = true;
-  /* We sleep here a bit to prevent invalidation immediately
-     after the selection change before we have started processing
-     the mail. */
-  Sleep (250);
+  gpgrt_lock_lock(&invalidate_lock);
+
+  int sleep_ms = (intptr_t)minsleep;
+  Sleep (sleep_ms);
+  int i = 0;
+  while (invalidation_blocked)
+    {
+      i++;
+      if (i % 10 == 0)
+        {
+          log_debug ("%s:%s: Waiting for invalidation.",
+                     SRCNAME, __func__);
+        }
+
+      Sleep (100);
+      /* Do we need an abort statement here? */
+    }
   do_in_ui_thread (INVALIDATE_UI, nullptr);
+  TRACEPOINT;
   invalidation_in_progress = false;
   gpgrt_lock_unlock(&invalidate_lock);
   return 0;
@@ -410,3 +483,26 @@ close_mail (LPVOID mail)
   do_in_ui_thread (CLOSE, mail);
   return 0;
 }
+
+void
+blockInv()
+{
+  invalidation_blocked++;
+  log_oom_extra ("%s:%s: Invalidation block count %i",
+                 SRCNAME, __func__, invalidation_blocked);
+}
+
+void
+unblockInv()
+{
+  invalidation_blocked--;
+  log_oom_extra ("%s:%s: Invalidation block count %i",
+                 SRCNAME, __func__, invalidation_blocked);
+
+  if (invalidation_blocked < 0)
+    {
+      log_error ("%s:%s: Invalidation block mismatch",
+                 SRCNAME, __func__);
+      invalidation_blocked = 0;
+    }
+}
