@@ -53,6 +53,8 @@ typedef enum _gpgol_wmsg_type
   INVALIDATE_LAST_MAIL,
   REVERT_MAIL,
   CLEAR_REPLY_FORWARD,
+  DO_AUTO_SECURE,
+  DONT_AUTO_SECURE,
 } gpgol_wmsg_type;
 
 typedef struct
@@ -68,11 +70,6 @@ typedef struct
 HWND
 create_responder_window ();
 
-/** Send a message to the UI thread through the responder Window.
-  Returns 0 on success. */
-int
-send_msg_to_ui_thread (wm_ctx_t *ctx);
-
 /** Uses send_msg_to_ui_thread to execute the request
   in the ui thread.  Returns the result. */
 int
@@ -90,16 +87,19 @@ do_in_ui_thread_async (gpgol_wmsg_type type, void *data, int delay = 0);
 HHOOK
 create_message_hook();
 
+/** Block ui invalidation. The idea here is to further reduce
+    UI invalidations because depending on timing they might crash.
+    So we try to block invalidation for as long as it is a bad time
+    for us. */
+void blockInv ();
+
+/** Unblock ui invalidation */
+void unblockInv ();
+
 DWORD WINAPI
-delayed_invalidate_ui (LPVOID);
+delayed_invalidate_ui (LPVOID minsleep_ms = 0);
 
 DWORD WINAPI
 close_mail (LPVOID);
 
-void add_explorer (LPDISPATCH explorer);
-void remove_explorer (LPDISPATCH explorer);
-
-/* The lock to invalide the ui */
-extern gpgrt_lock_t invalidate_lock;
-
 #endif // WINDOWMESSAGES_H
