@@ -300,7 +300,7 @@ WKSHelper::start_check (const std::string &mbox, bool forced) const
   log_debug ("%s:%s: WKSHelper starting check",
              SRCNAME, __func__);
   /* Start the actual work that can be done in a background thread. */
-  CloseHandle (CreateThread (nullptr, 0, do_check, strdup (mbox.c_str ()), 0,
+  CloseHandle (CreateThread (nullptr, 0, do_check, xstrdup (mbox.c_str ()), 0,
                              nullptr));
   return;
 }
@@ -324,8 +324,14 @@ WKSHelper::load () const
         }
 
       WKSState state = (WKSState) strtol (states[0].c_str (), nullptr, 10);
-      time_t update_time = (time_t) strtol (states[1].c_str (), nullptr, 10);
+      if (state == PublishInProgress)
+        {
+          /* Probably an error during the last publish. Let's start again. */
+          update_state (mbox, NotChecked, false);
+          continue;
+        }
 
+      time_t update_time = (time_t) strtol (states[1].c_str (), nullptr, 10);
       update_state (mbox, state, false);
       update_last_checked (mbox, update_time, false);
     }
@@ -380,7 +386,7 @@ WKSHelper::allow_notify (int sleepTimeMS) const
       if (pair.second == ConfirmationSeen ||
           pair.second == NeedsPublish)
         {
-          auto *args = new std::pair<char *, int> (strdup (pair.first.c_str()),
+          auto *args = new std::pair<char *, int> (xstrdup (pair.first.c_str()),
                                                    sleepTimeMS);
           CloseHandle (CreateThread (nullptr, 0, do_notify,
                                      args, 0,
@@ -400,12 +406,16 @@ WKSHelper::notify (const char *cBox) const
 
   if (state == NeedsPublish)
     {
+      char *buf;
+      gpgrt_asprintf (&buf, _("A Pubkey directory is available for the address:\n\n"
+                              "\t%s\n\n"
+                              "Register your Pubkey in that directory to make\n"
+                              "it easy for others to send you encrypted mail.\n\n"
+                              "It's secure and free!\n\n"
+                              "Register automatically?"), mbox.c_str ());
+      memdbg_alloc (buf);
       if (gpgol_message_box (get_active_hwnd (),
-                             _("A Pubkey directory is available for your domain.\n\n"
-                               "Register your Pubkey in that directory to make\n"
-                               "it easy for others to send you encrypted mail.\n\n"
-                               "It's secure and free!\n\n"
-                               "Register automatically?"),
+                             buf,
                              _("GpgOL: Pubkey directory available!"), MB_YESNO) == IDYES)
         {
           start_publish (mbox);
@@ -414,6 +424,7 @@ WKSHelper::notify (const char *cBox) const
         {
           update_state (mbox, PublishDenied);
         }
+      xfree (buf);
       return;
     }
   if (state == ConfirmationSeen)
@@ -433,6 +444,7 @@ WKSHelper::start_publish (const std::string &mbox) const
   log_debug ("%s:%s: Start publish for '%s'",
              SRCNAME, __func__, mbox.c_str ());
 
+  update_state (mbox, PublishInProgress);
   const auto key = GpgME::Key::locate (mbox.c_str ());
 
   if (key.isNull ())
@@ -505,6 +517,8 @@ WKSHelper::start_publish (const std::string &mbox) const
                            "your provider to finish the registration."),
                          _("GpgOL: Registration request sent!"), MB_OK);
     }
+
+  update_state (mbox, RequestSent);
   return;
 }
 
@@ -594,6 +608,23 @@ WKSHelper::send_mail (const std::string &mimeData) const
       return -1;
     }
 
+  /* Now we have a problem. The created LPDISPATCH pointer has
+     a different value then the one with which we saw the ItemLoad
+     event. But we want to get the mail object. So,.. surpise
+     a Hack! :-) */
+  auto last_mail = Mail::getLastMail ();
+
+  if (!Mail::isValidPtr (last_mail))
+    {
+      log_error ("%s:%s: Invalid last mail %p.",
+                 SRCNAME, __func__, last_mail);
+      return -1;
+    }
+  /* Adding to / Subject etc already leads to changes and events so
+     we set up the state before this. */
+  last_mail->setOverrideMIMEData (mimeData);
+  last_mail->setCryptState (Mail::NeedsSecondAfterWrite);
+
   if (put_oom_string (mail, "Subject", subject.c_str ()))
     {
       TRACEPOINT;
@@ -617,21 +648,6 @@ WKSHelper::send_mail (const std::string &mimeData) const
     }
   gpgol_release (account);
 
-  /* Now we have a problem. The created LPDISPATCH pointer has
-     a different value then the one with which we saw the ItemLoad
-     event. But we want to get the mail object. So,.. surpise
-     a Hack! :-) */
-  auto last_mail = Mail::get_last_mail ();
-
-  if (!Mail::is_valid_ptr (last_mail))
-    {
-      log_error ("%s:%s: Invalid last mail %p.",
-                 SRCNAME, __func__, last_mail);
-      return -1;
-    }
-  last_mail->set_override_mime_data (mimeData);
-  last_mail->set_crypt_state (Mail::NeedsSecondAfterWrite);
-
   if (invoke_oom_method (mail, "Save", nullptr))
     {
       // Should not happen.
@@ -675,16 +691,16 @@ WKSHelper::handle_confirmation_notify (const std::string &mbox) const
   GpgME::Data *mimeData = pair.first;
   Mail *mail = pair.second;
 
-  if (!mail)
+  if (!mail && !mimeData)
     {
-      log_debug ("%s:%s: Confirmation notify without cached mail.",
+      log_debug ("%s:%s: Confirmation notify without cached data.",
                  SRCNAME, __func__);
-    }
 
-  if (!mimeData)
-    {
-      log_error ("%s:%s: Confirmation notify without cached data.",
-                 SRCNAME, __func__);
+      /* This happens when we have seen a confirmation but have
+       * not confirmed it and the state was saved. So we go back
+       * to the confirmation sent state and wait until we see
+       * the confirmation the next time. */
+      update_state (mbox, ConfirmationSent);
       return;
     }
 
@@ -760,7 +776,7 @@ WKSHelper::handle_confirmation_notify (const std::string &mbox) const
                         _("GpgOL: Request confirmed!"), MB_OK);
    }
 
-  if (mail && Mail::is_valid_ptr (mail))
+  if (mail && Mail::isValidPtr (mail))
     {
       invoke_oom_method (mail->item(), "Delete", nullptr);
     }
@@ -782,21 +798,19 @@ WKSHelper::handle_confirmation_read (Mail *mail, LPSTREAM stream) const
     }
 
   /* Get the recipient of the confirmation mail */
-  char **recipients = mail->get_recipients ();
+  const auto recipients = mail->getRecipients_o ();
 
   /* We assert that we have one recipient as the mail should have been
      sent by the wks-server. */
-  if (!recipients || !recipients[0] || recipients[1])
+  if (recipients.size() != 1)
     {
       log_error ("%s:%s: invalid recipients",
                  SRCNAME, __func__);
-      release_cArray (recipients);
       gpgol_release (stream);
       return;
     }
 
   std::string mbox = recipients[0];
-  release_cArray (recipients);
 
   /* Prepare stdin for the wks-client process */
 
